@@ -65,6 +65,8 @@
 #include "src/api/step_ctx.h"
 #include "src/api/pmi_server.h"
 
+#define STEP_ABORT_TIME 2
+
 extern char **environ;
 
 /**********************************************************************
@@ -252,7 +254,7 @@ int slurm_step_launch (slurm_step_ctx_t *ctx, char *launcher_host,
 		ctx->launch_state->io.normal =
 			client_io_handler_create(params->local_fds,
 						 ctx->step_req->num_tasks,
-						 ctx->step_req->node_count,
+						 launch.nnodes,
 						 ctx->step_resp->cred,
 						 params->labelio);
 		if (ctx->launch_state->io.normal == NULL) {
@@ -315,7 +317,8 @@ int slurm_step_launch_wait_start(slurm_step_ctx_t *ctx)
 		if (sls->abort) {
 			if (!sls->abort_action_taken) {
 				slurm_kill_job_step(ctx->job_id,
-						    ctx->step_resp->job_step_id,
+						    ctx->step_resp->
+						    job_step_id,
 						    SIGKILL);
 				sls->abort_action_taken = true;
 			}
@@ -364,20 +367,22 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 		} else {
 			if (!sls->abort_action_taken) {
 				slurm_kill_job_step(ctx->job_id,
-						    ctx->step_resp->job_step_id,
+						    ctx->step_resp->
+						    job_step_id,
 						    SIGKILL);
 				sls->abort_action_taken = true;
 			}
 			if (!time_set) {
-				/* Only set the time once, because we only
-				 * want to wait 10 seconds, no matter how many
+				/* Only set the time once, because we only want
+				 * to wait STEP_ABORT_TIME, no matter how many
 				 * times the condition variable is signalled.
 				 */
-				ts.tv_sec = time(NULL) + 10;
+				ts.tv_sec = time(NULL) + STEP_ABORT_TIME;
 				time_set = true;
 				/* FIXME - should this be a callback? */
 				info("Job step aborted: Waiting up to "
-				     "10 seconds for job step to finish.");
+				     "%d seconds for job step to finish.",
+				     STEP_ABORT_TIME);
 			}
 
 			errnum = pthread_cond_timedwait(&sls->cond,
@@ -396,14 +401,18 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 					ctx->job_id,
 					ctx->step_resp->job_step_id,
 					SIGKILL);
-				if (!sls->user_managed_io)
-					client_io_handler_abort(sls->io.normal);
+				if (!sls->user_managed_io) {
+					client_io_handler_abort(sls->
+								io.normal);
+				}
 				break;
 			} else if (errnum != 0) {
 				error("Error waiting on condition in"
 				      " slurm_step_launch_wait_finish: %m");
-				if (!sls->user_managed_io)
-					client_io_handler_abort(sls->io.normal);
+				if (!sls->user_managed_io) {
+					client_io_handler_abort(sls->
+								io.normal);
+				}
 				break;
 			}
 		}
@@ -510,7 +519,7 @@ void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 	
 	debug3("sending signal to host %s", name);
 	
-	if (!(ret_list = slurm_send_recv_msgs(name, &req, 0))) { 
+	if (!(ret_list = slurm_send_recv_msgs(name, &req, 0, false))) { 
 		error("fwd_signal: slurm_send_recv_msgs really failed bad");
 		xfree(name);
 		return;
@@ -562,6 +571,7 @@ struct step_launch_state *step_launch_state_create(slurm_step_ctx_t *ctx)
 	sls->resp_port = NULL;
 	sls->abort = false;
 	sls->abort_action_taken = false;
+	sls->no_kill = ctx->no_kill;
 	sls->mpi_info->jobid = ctx->step_req->job_id;
 	sls->mpi_info->stepid = ctx->step_resp->job_step_id;
 	sls->mpi_info->step_layout = layout;
@@ -624,7 +634,8 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 	sls->resp_port = xmalloc(sizeof(uint16_t) * sls->num_resp_port);
 	for (i = 0; i < sls->num_resp_port; i++) {
 		if (net_stream_listen(&sock, &port) < 0) {
-			error("unable to intialize step launch listening socket: %m");
+			error("unable to intialize step launch listening "
+			      "socket: %m");
 			return SLURM_ERROR;
 		}
 		sls->resp_port[i] = port;
@@ -767,6 +778,13 @@ _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 	task_exit_msg_t *msg = (task_exit_msg_t *) exit_msg->data;
 	int i;
 
+	if ((msg->job_id != sls->mpi_info->jobid) || 
+	    (msg->step_id != sls->mpi_info->stepid)) {
+		debug("Received MESSAGE_TASK_EXIT from wrong job: %u.%u",
+		      msg->job_id, msg->step_id);
+		return;
+	}
+
 	/* Record SIGTERM and SIGKILL termination codes to 
 	 * recognize abnormal termination */
 	if (WIFSIGNALED(msg->return_code)) {
@@ -838,6 +856,15 @@ _node_fail_handler(struct step_launch_state *sls, slurm_msg_t *fail_msg)
 	int node_id, num_tasks;
 
 	error("Node failure on %s", nf->nodelist);
+	if (!sls->no_kill) {
+		info("Cancelling job step %u.%u", nf->job_id, nf->step_id);
+		slurm_kill_job_step(nf->job_id, nf->step_id, SIGKILL);
+		/* In an ideal world, we close the socket to this node and
+		 * normally terminate the remaining tasks. In practice this
+		 * is very difficult. The exercise is left to the reader. */
+		exit(1);
+	}
+
 	fail_nodes = hostset_create(nf->nodelist);
 	fail_itr = hostset_iterator_create(fail_nodes);
 	num_node_ids = hostset_count(fail_nodes);
@@ -1032,7 +1059,7 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 	
 	if(!(ret_list = slurm_send_recv_msgs(
 		     ctx->step_resp->step_layout->node_list,
-		     &msg, timeout))) {
+		     &msg, timeout, false))) {
 		error("slurm_send_recv_msgs failed miserably: %m");
 		return SLURM_ERROR;
 	}
