@@ -339,7 +339,7 @@ static int _get_local_association_list(void *db_conn, int enforce)
 	if(local_cluster_name) {
 		assoc_q.cluster_list = list_create(NULL);
 		list_append(assoc_q.cluster_list, local_cluster_name);
-	} else if(enforce && !slurmdbd_conf) {
+	} else if((enforce & ACCOUNTING_ENFORCE_ASSOCS) && !slurmdbd_conf) {
 		error("_get_local_association_list: "
 		      "no cluster name here going to get "
 		      "all associations.");
@@ -356,9 +356,10 @@ static int _get_local_association_list(void *db_conn, int enforce)
 	if(!local_association_list) {
 		/* create list so we don't keep calling this if there
 		   isn't anything there */
-		local_association_list = list_create(NULL);
+		local_association_list = 
+			list_create(destroy_acct_association_rec);
 		slurm_mutex_unlock(&assoc_mgr_association_lock);
-		if(enforce) {
+		if(enforce & ACCOUNTING_ENFORCE_ASSOCS) {
 			error("_get_local_association_list: "
 			      "no list was made.");
 			return SLURM_ERROR;
@@ -387,7 +388,7 @@ static int _get_local_qos_list(void *db_conn, int enforce)
 
 	if(!local_qos_list) {
 		slurm_mutex_unlock(&local_qos_lock);
-		if(enforce) {
+		if(enforce & ACCOUNTING_ENFORCE_ASSOCS) {
 			error("_get_local_qos_list: no list was made.");
 			return SLURM_ERROR;
 		} else {
@@ -414,7 +415,7 @@ static int _get_local_user_list(void *db_conn, int enforce)
 
 	if(!local_user_list) {
 		slurm_mutex_unlock(&local_user_lock);
-		if(enforce) {
+		if(enforce & ACCOUNTING_ENFORCE_ASSOCS) {
 			error("_get_local_user_list: "
 			      "no list was made.");
 			return SLURM_ERROR;
@@ -443,7 +444,7 @@ static int _get_local_wckey_list(void *db_conn, int enforce)
 	if(local_cluster_name) {
 		wckey_q.cluster_list = list_create(NULL);
 		list_append(wckey_q.cluster_list, local_cluster_name);
-	} else if(enforce && !slurmdbd_conf) {
+	} else if((enforce & ACCOUNTING_ENFORCE_WCKEYS) && !slurmdbd_conf) {
 		error("_get_local_wckey_list: "
 		      "no cluster name here going to get "
 		      "all wckeys.");
@@ -460,9 +461,9 @@ static int _get_local_wckey_list(void *db_conn, int enforce)
 	if(!assoc_mgr_wckey_list) {
 		/* create list so we don't keep calling this if there
 		   isn't anything there */
-		assoc_mgr_wckey_list = list_create(NULL);
+		assoc_mgr_wckey_list = list_create(destroy_acct_wckey_rec);
 		slurm_mutex_unlock(&assoc_mgr_wckey_lock);
-		if(enforce) {
+		if(enforce & ACCOUNTING_ENFORCE_WCKEYS) {
 			error("_get_local_wckey_list: "
 			      "no list was made.");
 			return SLURM_ERROR;
@@ -494,7 +495,7 @@ static int _refresh_local_association_list(void *db_conn, int enforce)
 	if(local_cluster_name) {
 		assoc_q.cluster_list = list_create(NULL);
 		list_append(assoc_q.cluster_list, local_cluster_name);
-	} else if(enforce && !slurmdbd_conf) {
+	} else if((enforce & ACCOUNTING_ENFORCE_ASSOCS) && !slurmdbd_conf) {
 		error("_refresh_local_association_list: "
 		      "no cluster name here going to get "
 		      "all associations.");
@@ -634,7 +635,7 @@ static int _refresh_local_wckey_list(void *db_conn, int enforce)
 	if(local_cluster_name) {
 		wckey_q.cluster_list = list_create(NULL);
 		list_append(wckey_q.cluster_list, local_cluster_name);
-	} else if(enforce && !slurmdbd_conf) {
+	} else if((enforce & ACCOUNTING_ENFORCE_WCKEYS) && !slurmdbd_conf) {
 		error("_refresh_local_wckey_list: "
 		      "no cluster name here going to get "
 		      "all wckeys.");
@@ -745,7 +746,7 @@ extern int assoc_mgr_fill_in_assoc(void *db_conn, acct_association_rec_t *assoc,
 			return SLURM_ERROR;
 	}
 	if((!local_association_list || !list_count(local_association_list))
-	   && !enforce) 
+	   && !(enforce & ACCOUNTING_ENFORCE_ASSOCS)) 
 		return SLURM_SUCCESS;
 
 	if(!assoc->id) {
@@ -753,7 +754,7 @@ extern int assoc_mgr_fill_in_assoc(void *db_conn, acct_association_rec_t *assoc,
 			acct_user_rec_t user;
 
 			if(assoc->uid == (uint32_t)NO_VAL) {
-				if(enforce) {
+				if(enforce & ACCOUNTING_ENFORCE_ASSOCS) {
 					error("get_assoc_id: "
 					      "Not enough info to "
 					      "get an association");
@@ -766,7 +767,7 @@ extern int assoc_mgr_fill_in_assoc(void *db_conn, acct_association_rec_t *assoc,
 			user.uid = assoc->uid;
 			if(assoc_mgr_fill_in_user(db_conn, &user, enforce) 
 			   == SLURM_ERROR) {
-				if(enforce) 
+				if(enforce & ACCOUNTING_ENFORCE_ASSOCS) 
 					return SLURM_ERROR;
 				else {
 					return SLURM_SUCCESS;
@@ -835,7 +836,7 @@ extern int assoc_mgr_fill_in_assoc(void *db_conn, acct_association_rec_t *assoc,
 	
 	if(!ret_assoc) {
 		slurm_mutex_unlock(&assoc_mgr_association_lock);
-		if(enforce) 
+		if(enforce & ACCOUNTING_ENFORCE_ASSOCS) 
 			return SLURM_ERROR;
 		else
 			return SLURM_SUCCESS;
@@ -895,7 +896,8 @@ extern int assoc_mgr_fill_in_user(void *db_conn, acct_user_rec_t *user,
 		if(_get_local_user_list(db_conn, enforce) == SLURM_ERROR)
 			return SLURM_ERROR;
 
-	if((!local_user_list || !list_count(local_user_list)) && !enforce) 
+	if((!local_user_list || !list_count(local_user_list)) 
+	   && !(enforce & ACCOUNTING_ENFORCE_ASSOCS)) 
 		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&local_user_lock);
@@ -940,7 +942,7 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 			return SLURM_ERROR;
 	}
 	if((!assoc_mgr_wckey_list || !list_count(assoc_mgr_wckey_list))
-	   && !enforce) 
+	   && !(enforce & ACCOUNTING_ENFORCE_WCKEYS)) 
 		return SLURM_SUCCESS;
 
 	if(!wckey->id) {
@@ -948,7 +950,7 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 			acct_user_rec_t user;
 
 			if(wckey->uid == (uint32_t)NO_VAL && !wckey->user) {
-				if(enforce) {
+				if(enforce & ACCOUNTING_ENFORCE_WCKEYS) {
 					error("get_wckey_id: "
 					      "Not enough info to "
 					      "get an wckey");
@@ -962,7 +964,7 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 			user.name = wckey->user;
 			if(assoc_mgr_fill_in_user(db_conn, &user, enforce) 
 			   == SLURM_ERROR) {
-				if(enforce) 
+				if(enforce & ACCOUNTING_ENFORCE_WCKEYS) 
 					return SLURM_ERROR;
 				else {
 					return SLURM_SUCCESS;
@@ -972,7 +974,7 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 				wckey->user = user.name;
 			wckey->name = user.default_wckey;
 		} else if(wckey->uid == (uint32_t)NO_VAL && !wckey->user) {
-			if(enforce) {
+			if(enforce & ACCOUNTING_ENFORCE_WCKEYS) {
 				error("get_wckey_id: "
 				      "Not enough info 2 to "
 				      "get an wckey");
@@ -1017,13 +1019,22 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 				       wckey->name, found_wckey->name);
 				continue;
 			}
-			
+
 			/* only check for on the slurmdbd */
-			if(!local_cluster_name && found_wckey->cluster
-			   && strcasecmp(wckey->cluster,
-					 found_wckey->cluster)) {
-				debug4("not the right cluster");
-				continue;
+			if(!local_cluster_name) {
+				if(!wckey->cluster) {
+					error("No cluster name was given "
+					      "to check against, "
+					      "we need one to get a wckey.");
+					continue;
+				}
+				
+				if(found_wckey->cluster
+				   && strcasecmp(wckey->cluster, 
+						 found_wckey->cluster)) {
+					debug4("not the right cluster");
+					continue;
+				}
 			}
 		}
 		ret_wckey = found_wckey;
@@ -1033,7 +1044,7 @@ extern int assoc_mgr_fill_in_wckey(void *db_conn, acct_wckey_rec_t *wckey,
 	
 	if(!ret_wckey) {
 		slurm_mutex_unlock(&assoc_mgr_wckey_lock);
-		if(enforce) 
+		if(enforce & ACCOUNTING_ENFORCE_WCKEYS) 
 			return SLURM_ERROR;
 		else
 			return SLURM_SUCCESS;
@@ -1137,6 +1148,7 @@ extern int assoc_mgr_update_local_assocs(acct_update_object_t *update)
 	ListIterator itr = NULL;
 	int rc = SLURM_SUCCESS;
 	int parents_changed = 0;
+	List remove_list = NULL;
 
 	if(!local_association_list)
 		return SLURM_SUCCESS;
@@ -1276,9 +1288,19 @@ extern int assoc_mgr_update_local_assocs(acct_update_object_t *update)
 				//rc = SLURM_ERROR;
 				break;
 			}
-			if (remove_assoc_notify)
-				remove_assoc_notify(rec);
-			list_delete_item(itr);
+			if(remove_assoc_notify) {
+				/* since there are some deadlock
+				   issues while inside our lock here
+				   we have to process a notify later 
+				*/
+				if(!remove_list)
+					remove_list = list_create(
+						destroy_acct_association_rec);
+				list_remove(itr);
+				list_append(remove_list, rec);
+			} else
+				list_delete_item(itr);
+
 			break;
 		default:
 			break;
@@ -1330,6 +1352,18 @@ extern int assoc_mgr_update_local_assocs(acct_update_object_t *update)
 
 	list_iterator_destroy(itr);
 	slurm_mutex_unlock(&assoc_mgr_association_lock);
+	
+	/* This needs to happen outside of the
+	   assoc_mgr_association_lock */
+	if(remove_list) {
+		itr = list_iterator_create(remove_list);
+
+		while((rec = list_next(itr))) 
+			remove_assoc_notify(rec);
+		
+		list_iterator_destroy(itr);
+		list_destroy(remove_list);
+	}
 
 	return rc;	
 }
@@ -1620,7 +1654,7 @@ extern int assoc_mgr_validate_assoc_id(void *db_conn,
 			return SLURM_ERROR;
 
 	if((!local_association_list || !list_count(local_association_list))
-	   && !enforce) 
+	   && !(enforce & ACCOUNTING_ENFORCE_ASSOCS)) 
 		return SLURM_SUCCESS;
 	
 	slurm_mutex_lock(&assoc_mgr_association_lock);
@@ -1632,7 +1666,7 @@ extern int assoc_mgr_validate_assoc_id(void *db_conn,
 	list_iterator_destroy(itr);
 	slurm_mutex_unlock(&assoc_mgr_association_lock);
 
-	if(found_assoc || !enforce)
+	if(found_assoc || !(enforce & ACCOUNTING_ENFORCE_ASSOCS))
 		return SLURM_SUCCESS;
 
 	return SLURM_ERROR;
@@ -1836,6 +1870,8 @@ extern int load_assoc_mgr_state(char *state_save_location)
 				break;
 			}
 			slurm_mutex_lock(&assoc_mgr_association_lock);
+			if(local_association_list)
+				list_destroy(local_association_list);
 			local_association_list = msg->my_list;
 			_post_association_list(local_association_list);
 			debug("Recovered %u associations", 
@@ -1854,6 +1890,8 @@ extern int load_assoc_mgr_state(char *state_save_location)
 				break;
 			}
 			slurm_mutex_lock(&local_user_lock);
+			if(local_user_list)
+				list_destroy(local_user_list);
 			local_user_list = msg->my_list;
 			_post_user_list(local_user_list);
 			debug("Recovered %u users", 
@@ -1872,6 +1910,8 @@ extern int load_assoc_mgr_state(char *state_save_location)
 				break;
 			}
 			slurm_mutex_lock(&local_qos_lock);
+			if(local_qos_list)
+				list_destroy(local_qos_list);
 			local_qos_list = msg->my_list;
 			debug("Recovered %u qos", 
 			      list_count(local_qos_list));
@@ -1889,6 +1929,8 @@ extern int load_assoc_mgr_state(char *state_save_location)
 				break;
 			}
 			slurm_mutex_lock(&assoc_mgr_wckey_lock);
+			if(assoc_mgr_wckey_list)
+				list_destroy(assoc_mgr_wckey_list);
 			assoc_mgr_wckey_list = msg->my_list;
 			debug("Recovered %u wckeys", 
 			      list_count(assoc_mgr_wckey_list));
