@@ -267,7 +267,6 @@ int setup_env(env_t *env)
 	int rc = SLURM_SUCCESS;
 	char *dist = NULL;
 	char *lllp_dist = NULL;
-	char *bgl_part_id = NULL;
 	char addrbuf[INET_ADDRSTRLEN];
 
 	if (env == NULL)
@@ -529,10 +528,22 @@ int setup_env(env_t *env)
 		rc = SLURM_FAILURE;
 	}
 
+#ifdef HAVE_BG
 	if(env->select_jobinfo) {
+		char *bgl_part_id = NULL;
 		select_g_get_jobinfo(env->select_jobinfo, 
 				     SELECT_DATA_BLOCK_ID, &bgl_part_id);
 		if (bgl_part_id) {
+#ifndef HAVE_BGL
+			uint16_t conn_type = (uint16_t)NO_VAL;
+			select_g_get_jobinfo(env->select_jobinfo, 
+					     SELECT_DATA_CONN_TYPE, &conn_type);
+			if(conn_type > SELECT_SMALL) {
+				if(setenvf(&env->env, 
+					   "SUBMIT_POOL", "%s", bgl_part_id))
+					rc = SLURM_FAILURE;
+			}
+#endif
 			if(setenvf(&env->env, 
 				   "MPIRUN_PARTITION", "%s", bgl_part_id))
 				rc = SLURM_FAILURE;
@@ -541,15 +552,17 @@ int setup_env(env_t *env)
 				rc = SLURM_FAILURE;
 			if(setenvf(&env->env, "MPIRUN_NOALLOCATE", "%d", 1))
 				rc = SLURM_FAILURE;
+			xfree(bgl_part_id);
 		} else 
 			rc = SLURM_FAILURE;
 		
 		if(rc == SLURM_FAILURE)
 			error("Can't set MPIRUN_PARTITION "
 			      "environment variable");
-		xfree(bgl_part_id);
+		
 	}
-	
+#endif
+
 	if (env->jobid >= 0
 	    && setenvf(&env->env, "SLURM_JOBID", "%d", env->jobid)) {
 		error("Unable to set SLURM_JOBID environment");
@@ -786,7 +799,7 @@ void
 env_array_for_job(char ***dest, const resource_allocation_response_msg_t *alloc,
 		  const job_desc_msg_t *desc)
 {
-	char *bgl_part_id = NULL, *tmp;
+	char *tmp = NULL;
 	slurm_step_layout_t *step_layout = NULL;
 	uint32_t num_tasks = desc->num_tasks;
 
@@ -807,16 +820,28 @@ env_array_for_job(char ***dest, const resource_allocation_response_msg_t *alloc,
 	env_array_overwrite(dest, "LOADLBATCH", "yes");
 #endif
 
+#ifdef HAVE_BG
 	/* BlueGene only */
 	select_g_get_jobinfo(alloc->select_jobinfo, SELECT_DATA_BLOCK_ID,
-			     &bgl_part_id);
-	if (bgl_part_id) {
+			     &tmp);
+	if (tmp) {
+#ifndef HAVE_BGL
+		uint16_t conn_type = (uint16_t)NO_VAL;
+		select_g_get_jobinfo(alloc->select_jobinfo, 
+				     SELECT_DATA_CONN_TYPE, &conn_type);
+		if(conn_type > SELECT_SMALL) {
+			env_array_overwrite_fmt(dest, "SUBMIT_POOL", "%s",
+						tmp);
+		}
+#endif
 		env_array_overwrite_fmt(dest, "MPIRUN_PARTITION", "%s",
-					bgl_part_id);
+					tmp);
 		env_array_overwrite_fmt(dest, "MPIRUN_NOFREE", "%d", 1);
 		env_array_overwrite_fmt(dest, "MPIRUN_NOALLOCATE", "%d", 1);
-	}
 
+		xfree(tmp);
+	}
+#endif
 	/* OBSOLETE, but needed by MPI, do not remove */
 	env_array_overwrite_fmt(dest, "SLURM_JOBID", "%u", alloc->job_id);
 	env_array_overwrite_fmt(dest, "SLURM_NNODES", "%u", alloc->node_cnt);
