@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: hostlist.c 13672 2008-03-19 23:10:58Z jette $
+ *  $Id: hostlist.c 16088 2008-12-29 21:56:17Z jette $
  *****************************************************************************
  *  $LSDId: hostlist.c,v 1.14 2003/10/14 20:11:54 grondo Exp $
  *****************************************************************************
@@ -437,7 +437,7 @@ static char * _next_tok(char *sep, char **str)
 	char *tok;
 
 	/* push str past any leading separators */
-	while (**str != '\0' && strchr(sep, **str) != '\0')
+	while ((**str != '\0') && (strchr(sep, **str) != NULL))
 		(*str)++;
 
 	if (**str == '\0')
@@ -447,20 +447,34 @@ static char * _next_tok(char *sep, char **str)
 	tok = *str;
 
 	/* push str past token and leave pointing to first separator */
-	while (**str != '\0' && strchr(sep, **str) == '\0')
+	while ((**str != '\0') && (strchr(sep, **str) == NULL))
 		(*str)++;
 
-	/* if _single_ opening bracket exists b/w tok and str, push str
-	 * past first closing bracket */
-	if (   memchr(tok, '[', *str - tok) != NULL
-	&& memchr(tok, ']', *str - tok) == NULL ) {
+	/* if _single_ opening bracket exists b/w tok and str,
+	 * push str past first closing bracket */
+	if ((memchr(tok, '[', *str - tok) != NULL) &&
+	    (memchr(tok, ']', *str - tok) == NULL)) {
 		char *q = strchr(*str, ']');
-		if (q && memchr(*str, '[', q - *str) == NULL)
-			*str = q + 1;
+		if (q && (memchr(*str, '[', q - *str) == NULL))
+			*str = ++q;
+
+		/* push str past token and leave pointing to next separator */
+		while ((**str != '\0') && (strchr(sep, **str) == NULL))
+			(*str)++;
+
+		/* if _second_ opening bracket exists b/w tok and str,
+		 * push str past second closing bracket */
+		if ((**str != '\0') &&
+		    (memchr(tok, '[', *str - q) != NULL) &&
+		    (memchr(tok, ']', *str - q) == NULL)) {
+			q = strchr(*str, ']');
+			if (q && (memchr(*str, '[', q - *str) == NULL))
+				*str = q + 1;
+		}
 	}
 
 	/* nullify consecutive separators and push str beyond them */
-	while (**str != '\0' && strchr(sep, **str) != '\0')
+	while ((**str != '\0') && (strchr(sep, **str) != '\0'))
 		*(*str)++ = '\0';
 
 	return tok;
@@ -535,7 +549,7 @@ static int host_prefix_end(const char *hostname)
 	assert(hostname != NULL);
 
 	len = strlen(hostname);
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	if (len < 4)
 		return -1;
 	idx = len - 4;
@@ -873,7 +887,7 @@ static char *hostrange_pop(hostrange_t hr)
 		size = strlen(hr->prefix) + hr->width + 16;    
 		if (!(host = (char *) malloc(size * sizeof(char))))
 			out_of_memory("hostrange pop");
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (hr->width == 3) {
 			int coord[3];
 			coord[0] = hr->hi / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -914,7 +928,7 @@ static char *hostrange_shift(hostrange_t hr)
 		size = strlen(hr->prefix) + hr->width + 16;
 		if (!(host = (char *) malloc(size * sizeof(char))))
 			out_of_memory("hostrange shift");
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (hr->width == 3) {
 			int coord[3];
 			coord[0] = hr->lo / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -1016,19 +1030,48 @@ static hostrange_t hostrange_intersect(hostrange_t h1, hostrange_t h2)
  */
 static int hostrange_hn_within(hostrange_t hr, hostname_t hn)
 {
-	int retval = 0;
+	if (hr->singlehost) {
+		/*  
+		 *  If the current hostrange [hr] is a `singlehost' (no valid 
+		 *   numeric suffix (lo and hi)), then the hostrange [hr]
+		 *   stores just one host with name == hr->prefix.
+		 *  
+		 *  Thus the full hostname in [hn] must match hr->prefix, in
+		 *   which case we return true. Otherwise, there is no 
+		 *   possibility that [hn] matches [hr].
+		 */
+		if (strcmp (hn->hostname, hr->prefix) == 0)
+			return 1;
+		else 
+			return 0;
+	}
 
-	if (strcmp(hr->prefix, hn->prefix) == 0) {
-		if (!hostname_suffix_is_valid(hn)) {
-			if (hr->singlehost)
-			retval = 1;
-		} else if (hn->num <= hr->hi && hn->num >= hr->lo) {
+	/*
+	 *  Now we know [hr] is not a "singlehost", so hostname
+	 *   better have a valid numeric suffix, or there is no 
+	 *   way we can match
+	 */
+	if (!hostname_suffix_is_valid (hn))
+		return 0;
+
+	/*
+	 *  If hostrange and hostname prefixes don't match, then
+	 *   there is way the hostname falls within the range [hr].
+	 */
+	if (strcmp(hr->prefix, hn->prefix) != 0) 
+		return 0;
+
+	/*
+	 *  Finally, check whether [hn], with a valid numeric suffix,
+	 *   falls within the range of [hr].
+	 */
+	if (hn->num <= hr->hi && hn->num >= hr->lo) {
 			int width = hostname_suffix_width(hn);
 			int num = hn->num;
-			retval = _width_equiv(hr->lo, &hr->width, num, &width);
-		}
+		return (_width_equiv(hr->lo, &hr->width, num, &width));
 	}
-	return retval;
+
+	return 0;
 }
 
 
@@ -1054,7 +1097,7 @@ hostrange_to_string(hostrange_t hr, size_t n, char *buf, char *separator)
 	for (i = hr->lo; i <= hr->hi; i++) {
 		size_t m = (n - len) <= n ? n - len : 0; /* check for < 0 */
 		int ret = 0;
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (hr->width == 3) {
 			int coord[3];
 			coord[0] = i / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -1103,7 +1146,7 @@ static size_t hostrange_numstr(hostrange_t hr, size_t n, char *buf)
 	if (hr->singlehost || n == 0)
 		return 0;
 
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	if (hr->width == 3) {
 		int coord[3];
 		coord[0] = hr->lo / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -1121,7 +1164,7 @@ static size_t hostrange_numstr(hostrange_t hr, size_t n, char *buf)
 
 	if ((len >= 0) && (len < n) && (hr->lo < hr->hi)) {
 		int len2 = 0;
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (hr->width == 3) {
 			int coord[3];
 			coord[0] = hr->hi / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -1360,9 +1403,9 @@ hostlist_t _hostlist_create(const char *hostlist, char *sep, char *r_op)
 
 	if (hostlist == NULL)
 		return new;
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	fatal("WANT_RECKLESS_HOSTRANGE_EXPANSION does not work on "
-	      "bluegene systems!!!!");
+	      "Bluegene or Sun Constellation systems!!!!");
 #endif
 	orig = str = strdup(hostlist);
 	
@@ -1628,16 +1671,60 @@ static int _parse_range_list(char *str, struct _range *ranges, int len)
 	return count;
 }
 
-static void
-_push_range_list(hostlist_t hl, char *pfx, struct _range *rng,
-	int n)
+/* Validate prefix and push with the numeric suffix onto the hostlist
+ * The prefix can contain a up to one range expresseion (e.g. "rack[1-4]_").
+ * RET 0 on success, -1 on failure (invalid prefix) */
+static int
+_push_range_list(hostlist_t hl, char *prefix, struct _range *range,
+		 int n)
 {
-	int i;
-	
-	for (i = 0; i < n; i++) {
-		hostlist_push_hr(hl, pfx, rng->lo, rng->hi, rng->width);
-		rng++;
+	int i, k, nr;
+	char *p, *q;
+	char new_prefix[1024], tmp_prefix[1024];
+
+	strncpy(tmp_prefix, prefix, sizeof(tmp_prefix));
+	if (((p = strrchr(tmp_prefix, '[')) != NULL) &&
+	    ((q = strrchr(p, ']')) != NULL)) {
+		struct _range prefix_range[MAX_RANGES];
+		struct _range *saved_range = range, *pre_range = prefix_range;
+		unsigned long j, prefix_cnt = 0;
+		*p++ = '\0';
+		*q++ = '\0';
+		if (strrchr(tmp_prefix, '[') != NULL)
+			return -1;	/* third range is illegal */
+		nr = _parse_range_list(p, prefix_range, MAX_RANGES);
+		if (nr < 0)
+			return -1;	/* bad numeric expression */
+		for (i = 0; i < nr; i++) {
+			prefix_cnt += pre_range->hi - pre_range->lo + 1;
+			if (prefix_cnt > MAX_PREFIX_CNT) {
+				/* Prevent overflow of memory with user input
+				 * of something like "a[0-999999999].b[0-9]" */
+				return -1;
+			}
+			for (j = pre_range->lo; j <= pre_range->hi; j++) {
+				snprintf(new_prefix, sizeof(new_prefix),
+					 "%s%0*lu%s", tmp_prefix, 
+					 pre_range->width, j, q);
+				range = saved_range;
+				for (k = 0; k < n; k++) {
+					hostlist_push_hr(hl, new_prefix,
+							 range->lo, range->hi,
+							 range->width);
+					range++;
+				}
+			}
+			pre_range++;
+		}
+		return 0;
 	}
+
+	for (k = 0; k < n; k++) {
+		hostlist_push_hr(hl, prefix, 
+				 range->lo, range->hi, range->width);
+		range++;
+	}
+	return 0;
 }
 
 /*
@@ -1664,21 +1751,19 @@ _hostlist_create_bracketed(const char *hostlist, char *sep, char *r_op)
 	while ((tok = _next_tok(sep, &str)) != NULL) {
 		strncpy(cur_tok, tok, 1024);
 
-		if ((p = strchr(tok, '[')) != NULL) {
+		if ((p = strrchr(tok, '[')) != NULL) {
 			char *q, *prefix = tok;
 			*p++ = '\0';
 
 			if ((q = strchr(p, ']'))) {
-				if ((q[1] != ',') && (q[1] != '\0')) {
-					errno = EINVAL; /* Invalid suffix */
+				if ((q[1] != ',') && (q[1] != '\0'))
 					goto error;
-				}
 				*q = '\0';
 				nr = _parse_range_list(p, ranges, MAX_RANGES);
 				if (nr < 0) 
 					goto error;
-				_push_range_list(new, prefix, ranges, nr);
-
+				if (_push_range_list(new, prefix, ranges, nr))
+					goto error;
                 
 			} else {
 				/* The hostname itself contains a '['
@@ -1695,7 +1780,7 @@ _hostlist_create_bracketed(const char *hostlist, char *sep, char *r_op)
 	return new;
 
   error:
-	err = errno;
+	err = errno = EINVAL;
 	hostlist_destroy(new);
 	free(orig);
 	seterrno_ret(err, NULL);
@@ -2011,7 +2096,7 @@ _hostrange_string(hostrange_t hr, int depth)
 	int  len = snprintf(buf, MAXHOSTNAMELEN + 15, "%s", hr->prefix);
 
 	if (!hr->singlehost) {
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (hr->width == 3) {
 			int coord[3];
 			int temp = hr->lo+depth;
@@ -2251,6 +2336,7 @@ static void hostlist_coalesce(hostlist_t hl)
 /* attempt to join ranges at loc and loc-1 in a hostlist  */
 /* delete duplicates, return the number of hosts deleted  */
 /* assumes that the hostlist hl has been locked by caller */
+/* returns -1 if no range join occured */
 static int _attempt_range_join(hostlist_t hl, int loc)
 {
 	int ndup;
@@ -2290,7 +2376,7 @@ void hostlist_uniq(hostlist_t hl)
 }
 
 
-size_t hostlist_deranged_string(hostlist_t hl, size_t n, char *buf)
+ssize_t hostlist_deranged_string(hostlist_t hl, size_t n, char *buf)
 {
 	int i;
 	int len = 0;
@@ -2379,7 +2465,7 @@ _get_bracketed_list(hostlist_t hl, int *start, const size_t n, char *buf)
 	return len;
 }
 
-#ifdef HAVE_BG		
+#ifdef HAVE_BG
 
 /* logic for block node description */
 /* write the next bracketed hostlist, i.e. prefix[n-m,k,...]
@@ -2544,7 +2630,7 @@ _test_box(void)
 }
 #endif
 
-size_t hostlist_ranged_string(hostlist_t hl, size_t n, char *buf)
+ssize_t hostlist_ranged_string(hostlist_t hl, size_t n, char *buf)
 {
 	int i = 0;
 	int len = 0;
@@ -2561,9 +2647,9 @@ size_t hostlist_ranged_string(hostlist_t hl, size_t n, char *buf)
 		 * this does not necessarily contain a BlueGene
 		 * host list. It could just be numeric values */
 		if (hl->hr[0]->prefix[0]) {
-			error("This node is not in bluegene format.  "
-		 	     "The suffix was %d chars in length",
-		  	    hl->hr[0]->width);
+			debug("This node is not in bluegene format.  "
+		 	      "Prefix is %s and suffix is %d chars long",
+		  	      hl->hr[0]->prefix, hl->hr[0]->width);
 		}
 		goto notbox; 
 	}
@@ -2737,7 +2823,7 @@ char *hostlist_next(hostlist_iterator_t i)
 
 	len = snprintf(buf, MAXHOSTNAMELEN + 15, "%s", i->hr->prefix);
 	if (!i->hr->singlehost) {
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		if (i->hr->width == 3) {
 			int coord[3];
 			int temp = i->hr->lo + i->depth;
@@ -2857,12 +2943,14 @@ void hostset_destroy(hostset_t set)
 
 /* inserts a single range object into a hostset 
  * Assumes that the set->hl lock is already held
+ * Updates hl->nhosts
  */
 static int hostset_insert_range(hostset_t set, hostrange_t hr)
 {
-	int i, n = 0;
+	int i = 0;
 	int inserted = 0;
-	int retval = 0;
+	int nhosts = 0;
+	int ndups = 0;
 	hostlist_t hl;
 
 	hl = set->hl;
@@ -2870,24 +2958,25 @@ static int hostset_insert_range(hostset_t set, hostrange_t hr)
 	if (hl->size == hl->nranges && !hostlist_expand(hl))
 		return 0;
 
-	retval = hostrange_count(hr);
+	nhosts = hostrange_count(hr);
 
 	for (i = 0; i < hl->nranges; i++) {
 		if (hostrange_cmp(hr, hl->hr[i]) <= 0) {
-			n = hostrange_join(hr, hl->hr[i]);
 
-			if (n >= 0) {
+			if ((ndups = hostrange_join(hr, hl->hr[i])) >= 0) 
 				hostlist_delete_range(hl, i);
-				hl->nhosts -= n;
-			}
+			else if (ndups < 0)
+				ndups = 0;
 
 			hostlist_insert_range(hl, hr, i);
 
 			/* now attempt to join hr[i] and hr[i-1] */
 			if (i > 0) {
-				int m = _attempt_range_join(hl, i);
-				n += m;
+				int m;
+				if ((m = _attempt_range_join(hl, i)) > 0)
+					ndups += m;
 			}
+			hl->nhosts += nhosts - ndups;
 			inserted = 1;
 			break;
 		}
@@ -2895,10 +2984,17 @@ static int hostset_insert_range(hostset_t set, hostrange_t hr)
 
 	if (inserted == 0) {
 		hl->hr[hl->nranges++] = hostrange_copy(hr);
-		n = _attempt_range_join(hl, hl->nranges - 1);
+		hl->nhosts += nhosts;
+		if (hl->nranges > 1) {
+			if ((ndups = _attempt_range_join(hl, hl->nranges - 1)) <= 0)
+				ndups = 0;
+		}
 	}
 
-	return retval - n;
+	/*
+	 *  Return the number of unique hosts inserted
+	 */
+	return nhosts - ndups;
 }
 
 int hostset_insert(hostset_t set, const char *hosts)
@@ -2939,6 +3035,30 @@ static int hostset_find_host(hostset_t set, const char *host)
 	return retval;
 }
 
+int hostset_intersects(hostset_t set, const char *hosts)
+{
+	int retval = 0;
+	hostlist_t hl;
+	char *hostname;
+
+	assert(set->hl->magic == HOSTLIST_MAGIC);
+
+	hl = hostlist_create(hosts);
+	if (!hl)    /* malloc failure */
+		return retval;
+
+	while ((hostname = hostlist_pop(hl)) != NULL) {
+		retval += hostset_find_host(set, hostname);
+		free(hostname);
+		if (retval)
+			break;
+	}
+
+	hostlist_destroy(hl);
+
+	return retval;
+}
+
 int hostset_within(hostset_t set, const char *hosts)
 {
 	int nhosts, nfound;
@@ -2947,7 +3067,8 @@ int hostset_within(hostset_t set, const char *hosts)
 
 	assert(set->hl->magic == HOSTLIST_MAGIC);
 
-	hl = hostlist_create(hosts);
+	if (!(hl = hostlist_create(hosts)))
+        return (0);
 	nhosts = hostlist_count(hl);
 	nfound = 0;
 
@@ -2996,12 +3117,12 @@ int hostset_count(hostset_t set)
 	return hostlist_count(set->hl);
 }
 
-size_t hostset_ranged_string(hostset_t set, size_t n, char *buf)
+ssize_t hostset_ranged_string(hostset_t set, size_t n, char *buf)
 {
 	return hostlist_ranged_string(set->hl, n, buf);
 }
 
-size_t hostset_deranged_string(hostset_t set, size_t n, char *buf)
+ssize_t hostset_deranged_string(hostset_t set, size_t n, char *buf)
 {
 	return hostlist_deranged_string(set->hl, n, buf);
 }
