@@ -110,13 +110,15 @@ _slurm_mpi_context_create(const char *mpi_type)
 static int
 _slurm_mpi_context_destroy( slurm_mpi_context_t c )
 {
+	int rc = SLURM_SUCCESS;
+
 	/*
 	 * Must check return code here because plugins might still
 	 * be loaded and active.
 	 */
 	if ( c->plugin_list ) {
 		if ( plugrack_destroy( c->plugin_list ) != SLURM_SUCCESS ) {
-			return SLURM_ERROR;
+			rc = SLURM_ERROR;
 		}
 	} else {
 		plugin_unload(c->cur_plugin);
@@ -125,7 +127,7 @@ _slurm_mpi_context_destroy( slurm_mpi_context_t c )
 	xfree(c->mpi_type);
 	xfree(c);
 
-	return SLURM_SUCCESS;
+	return rc;
 }
 
 /*
