@@ -125,7 +125,7 @@ static void validate_and_set_defaults(slurm_ctl_conf_t *conf,
 				      s_p_hashtbl_t *hashtbl);
 
 s_p_options_t slurm_conf_options[] = {
-	{"AccountingStorageEnforce", S_P_UINT16},
+	{"AccountingStorageEnforce", S_P_STRING},
 	{"AccountingStorageHost", S_P_STRING},
 	{"AccountingStorageLoc", S_P_STRING},
 	{"AccountingStoragePass", S_P_STRING},
@@ -1764,6 +1764,10 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_string(&conf->mpi_default, "MpiDefault", hashtbl))
 		conf->mpi_default = xstrdup(DEFAULT_MPI_DEFAULT);
 
+	if(!s_p_get_boolean((bool *)&conf->track_wckey, 
+			    "TrackWCKey", hashtbl))
+		conf->track_wckey = false;
+
 	if (!s_p_get_string(&conf->accounting_storage_type,
 			    "AccountingStorageType", hashtbl)) {
 		if(default_storage_type)
@@ -1775,9 +1779,30 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 				xstrdup(DEFAULT_ACCOUNTING_STORAGE_TYPE);
 	}
 
-	if (!s_p_get_uint16(&conf->accounting_storage_enforce, 
-			    "AccountingStorageEnforce", hashtbl))
-		conf->accounting_storage_enforce = DEFAULT_ACCOUNTING_ENFORCE;
+	if (s_p_get_string(&temp_str, "AccountingStorageEnforce", hashtbl)) {
+		if (strstr(temp_str, "1") || strstr(temp_str, "associations"))
+			conf->accounting_storage_enforce 
+				|= ACCOUNTING_ENFORCE_ASSOCS;
+		if (strstr(temp_str, "2") || strstr(temp_str, "limits")) {
+			conf->accounting_storage_enforce 
+				|= ACCOUNTING_ENFORCE_ASSOCS;
+			conf->accounting_storage_enforce 
+				|= ACCOUNTING_ENFORCE_LIMITS;
+		}
+		if (strstr(temp_str, "wckeys")) {
+			conf->accounting_storage_enforce 
+				|= ACCOUNTING_ENFORCE_ASSOCS;
+			conf->accounting_storage_enforce 
+				|= ACCOUNTING_ENFORCE_WCKEYS;
+			conf->track_wckey = true;
+		}		
+		if (strstr(temp_str, "all")) {
+			conf->accounting_storage_enforce = 0xffff;
+			conf->track_wckey = true;
+		}		
+			
+		xfree(temp_str);
+	}
 
 	if (!s_p_get_string(&conf->accounting_storage_host,
 			    "AccountingStorageHost", hashtbl)) {
@@ -2040,10 +2065,6 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_uint16(&conf->wait_time, "WaitTime", hashtbl))
 		conf->wait_time = DEFAULT_WAIT_TIME;
 	
-	if(!s_p_get_boolean((bool *)&conf->track_wckey, 
-			    "TrackWCKey", hashtbl))
-		conf->track_wckey = false;
-
 	if (s_p_get_uint16(&conf->tree_width, "TreeWidth", hashtbl)) {
 		if (conf->tree_width == 0) {
 			error("TreeWidth=0 is invalid");
