@@ -280,13 +280,15 @@ slurm_auth_generic_errstr( int slurm_errno )
 static int
 _slurm_auth_context_destroy( slurm_auth_context_t c )
 {    
+	int rc = SLURM_SUCCESS;
+
         /*
          * Must check return code here because plugins might still
          * be loaded and active.
          */
         if ( c->plugin_list ) {
                 if ( plugrack_destroy( c->plugin_list ) != SLURM_SUCCESS ) {
-                        return SLURM_ERROR;
+                        rc =  SLURM_ERROR;
                 }
         } else {
 		plugin_unload(c->cur_plugin);
@@ -295,7 +297,7 @@ _slurm_auth_context_destroy( slurm_auth_context_t c )
         xfree( c->auth_type );
         xfree( c );
         
-        return SLURM_SUCCESS;
+        return rc;
 }
 
 int inline
