@@ -95,7 +95,12 @@ int inline slurm_set_api_config(slurm_protocol_config_t * protocol_conf);
  * returns a pointer to the current slurm_protocol_config object
  * RET slurm_protocol_config_t	- current slurm_protocol_config object
  */
-inline slurm_protocol_config_t *slurm_get_api_config();
+inline slurm_protocol_config_t *slurm_get_api_config(void);
+
+/* slurm_get_batch_start_timeout
+ * RET BatchStartTimeout value from slurm.conf
+ */
+uint16_t slurm_get_batch_start_timeout(void);
 
 /* slurm_get_def_mem_per_task
  * RET DefMemPerTask value from slurm.conf
@@ -169,6 +174,12 @@ uint16_t slurm_get_slurmdbd_port(void);
  */
 char *slurm_get_plugin_dir(void);
 
+/* slurm_get_private_data
+ * get private data from slurmctld_conf object
+ * RET uint16_t   - private_data
+ */
+uint16_t slurm_get_private_data(void);
+
 /* slurm_get_state_save_location
  * get state_save_location from slurmctld_conf object from slurmctld_conf object 
  * RET char *   - state_save_location directory, MUST be xfreed by caller
@@ -211,6 +222,11 @@ extern char *slurm_get_crypto_type(void);
  */
 extern uint16_t slurm_get_fast_schedule(void);
 
+/* slurm_get_track_wckey
+ * returns the value of track_wckey in slurmctld_conf object
+ */
+extern uint16_t slurm_get_track_wckey(void);
+
 /* slurm_set_tree_width
  * sets the value of tree_width in slurmctld_conf object
  * RET 0 or error code
@@ -239,6 +255,16 @@ char *slurm_get_accounting_storage_user(void);
  */
 char *slurm_get_accounting_storage_host(void);
 
+/* slurm_get_accounting_storage_enforce
+ * returns what level to enforce associations at
+ */
+int slurm_get_accounting_storage_enforce(void);
+
+/* slurm_get_is_association_based_accounting
+ * returns if we are doing accounting by associations
+ */
+int slurm_get_is_association_based_accounting(void);
+
 /* slurm_get_accounting_storage_pass
  * returns the storage password from slurmctld_conf object
  * RET char *    - storage location,  MUST be xfreed by caller
@@ -263,6 +289,12 @@ char *slurm_get_accounting_storage_pass(void);
  */
 uint32_t slurm_get_accounting_storage_port(void);
 
+/* slurm_set_accounting_storage_port
+ * sets the storage port in slurmctld_conf object
+ * RET 0 or error code
+ */
+int slurm_set_accounting_storage_port(uint32_t storage_port);
+
 /* slurm_get_jobacct_gather_type
  * returns the job accounting type from slurmctld_conf object
  * RET char *    - job accounting type,  MUST be xfreed by caller
@@ -311,6 +343,13 @@ char *slurm_get_jobcomp_pass(void);
  */
 uint32_t slurm_get_jobcomp_port(void);
 
+/* slurm_set_jobcomp_port
+ * sets the jobcomp port in slurmctld_conf object
+ * RET 0 or error code
+ */
+int slurm_set_jobcomp_port(uint32_t port);
+
+
 /* slurm_get_propagate_prio_process
  * return the PropagatePrioProcess flag from slurmctld_conf object
  */
@@ -778,11 +817,13 @@ int slurm_send_recv_node_msg(slurm_msg_t * request_msg,
  * IN nodelist	    - list of nodes to send to.
  * IN msg           - a slurm_msg struct to be sent by the function
  * IN timeout	    - how long to wait in milliseconds
+ * IN quiet       - if set, reduce logging details
  * RET List	    - List containing the responses of the childern
  *                    (if any) we forwarded the message to. List
  *                    containing type (ret_types_t).
  */
-List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout);
+List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout,
+			  bool quiet);
 
 /*
  *  Send a message to msg->address
