@@ -66,7 +66,12 @@ static int _clear_results(MYSQL *mysql_db)
 			      mysql_errno(mysql_db),
 			      mysql_error(mysql_db));
 	} while (rc == 0);
-	
+
+	if(rc > 0) {
+		errno = rc;
+		return SLURM_ERROR;
+	} 
+
 	return SLURM_SUCCESS;
 }
 
@@ -112,6 +117,7 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 				     storage_field_t *fields, char *ending)
 {
 	char *query = NULL;
+	char *correct_query = NULL;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	int i = 0;
@@ -162,6 +168,7 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 
 	itr = list_iterator_create(columns);
 	query = xstrdup_printf("alter table %s", table_name);
+	correct_query = xstrdup_printf("alter table %s", table_name);
 	START_TIMER;
 	while(fields[i].name) {
 		int found = 0;
@@ -171,6 +178,9 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 				xstrfmtcat(query, " modify %s %s,",
 					   fields[i].name,
 					   fields[i].options);
+				xstrfmtcat(correct_query, " modify %s %s,",
+					   fields[i].name,
+					   fields[i].options);
 				list_delete_item(itr);
 				found = 1;
 				break;
@@ -186,6 +196,9 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 					   fields[i].name,
 					   fields[i].options,
 					   fields[i-1].name);
+				xstrfmtcat(correct_query, " modify %s %s,",
+					   fields[i].name,
+					   fields[i].options);
 			} else {
 				info("adding column %s at the beginning "
 				     "of table %s",
@@ -195,6 +208,9 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 				xstrfmtcat(query, " add %s %s first,",
 					   fields[i].name,
 					   fields[i].options);
+				xstrfmtcat(correct_query, " modify %s %s,",
+					   fields[i].name,
+					   fields[i].options);
 			}
 			adding = 1;
 		}
@@ -228,9 +244,13 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 		if(temp[end]) {
 			end++;
 			primary_key = xstrndup(temp, end);
-			if(old_primary)
+			if(old_primary) {
 				xstrcat(query, " drop primary key,");
+				xstrcat(correct_query, " drop primary key,");
+			}
 			xstrfmtcat(query, " add %s,",  primary_key);
+			xstrfmtcat(correct_query, " add %s,",  primary_key);
+			
 			xfree(primary_key);
 		}
 	}
@@ -251,16 +271,21 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 		if(temp[end]) {
 			end++;
 			unique_index = xstrndup(temp, end);
-			if(old_index)
+			if(old_index) {
 				xstrfmtcat(query, " drop index %s,",
 					   old_index);
+				xstrfmtcat(correct_query, " drop index %s,",
+					   old_index);
+			}
 			xstrfmtcat(query, " add %s,", unique_index);
+			xstrfmtcat(correct_query, " add %s,", unique_index);
 			xfree(unique_index);
 		}
 	}
 	xfree(old_index);
 
 	query[strlen(query)-1] = ';';
+	correct_query[strlen(correct_query)-1] = ';';
 	//info("%d query\n%s", __LINE__, query);
 
 	/* see if we have already done this definition */
@@ -287,6 +312,7 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 		char *query2 = NULL;
 	
 		debug("Table %s has changed.  Updating...", table_name);
+
 		if(mysql_db_query(mysql_db, query)) {
 			xfree(query);
 			return SLURM_ERROR;
@@ -298,15 +324,17 @@ static int _mysql_make_table_current(MYSQL *mysql_db, char *table_name,
 					"on duplicate key update "
 					"definition=\"%s\", mod_time=%d;",
 					table_defs_table, now, now,
-					table_name, query, query, now);
+					table_name, correct_query,
+					correct_query, now);
 		if(mysql_db_query(mysql_db, query2)) {
 			xfree(query2);
 			return SLURM_ERROR;
 		}
 		xfree(query2);
 	}
-	
+
 	xfree(query);
+	xfree(correct_query);
 	query = xstrdup_printf("make table current %s", table_name);
 	END_TIMER2(query);
 	xfree(query);
@@ -377,17 +405,18 @@ extern int mysql_get_db_connection(MYSQL **mysql_db, char *db_name,
 {
 	int rc = SLURM_SUCCESS;
 	bool storage_init = false;
-
+	
 	if(!(*mysql_db = mysql_init(*mysql_db)))
 		fatal("mysql_init failed: %s", mysql_error(*mysql_db));
 	else {
+		unsigned int my_timeout = 30;
 #ifdef MYSQL_OPT_RECONNECT
-{
 		my_bool reconnect = 1;
 		/* make sure reconnect is on */
 		mysql_options(*mysql_db, MYSQL_OPT_RECONNECT, &reconnect);
-}
 #endif
+		mysql_options(*mysql_db, MYSQL_OPT_CONNECT_TIMEOUT,
+			      (char *)&my_timeout);
 		while(!storage_init) {
 			if(!mysql_real_connect(*mysql_db, db_info->host,
 					       db_info->user, db_info->pass,
@@ -544,6 +573,16 @@ extern MYSQL_RES *mysql_db_query_ret(MYSQL *mysql_db, char *query, bool last)
 	return result;
 }
 
+extern int mysql_db_query_check_after(MYSQL *mysql_db, char *query)
+{
+	int rc = SLURM_SUCCESS;
+		
+	if((rc = mysql_db_query(mysql_db, query)) != SLURM_ERROR)  
+		rc = _clear_results(mysql_db);
+	
+	return rc;
+}
+
 extern int mysql_insert_ret_id(MYSQL *mysql_db, char *query)
 {
 	int new_id = 0;
