@@ -52,6 +52,16 @@
 #define BA_SYSTEM_DIMENSIONS 1
 #endif
 
+#define PASS_DENY_X 0x0001
+#define PASS_DENY_Y 0x0002
+#define PASS_DENY_Z 0x0004
+#define PASS_DENY_ALL 0x00ff
+
+#define PASS_FOUND_X 0x0100
+#define PASS_FOUND_Y 0x0200
+#define PASS_FOUND_Z 0x0400
+#define PASS_FOUND_ANY 0xff00
+
 extern bool _initialized;
 
 enum {X, Y, Z};
@@ -92,8 +102,11 @@ typedef struct {
 	int geometry[BA_SYSTEM_DIMENSIONS]; /* size of block in geometry */
 	char *linuximage;              /* LinuxImage for this block */
 	char *mloaderimage;            /* mloaderImage for this block */
-	bool passthrough;              /* filled in if there are
-					  passthroughs in the block created */
+	uint16_t deny_pass;            /* PASSTHROUGH_FOUND is set if there are
+					  passthroughs in the block
+					  created you can deny
+					  passthroughs by setting the
+					  appropriate bits*/
 	int procs;                     /* Number of Real processors in
 					  block */
 	char *ramdiskimage;            /* RamDiskImage for this block */
@@ -249,6 +262,7 @@ typedef struct {
 } ba_bp_map_t;
 
 /* Global */
+extern my_bluegene_t *bg;
 extern List bp_map_list; /* list used for conversion from XYZ to Rack
 			  * midplane */
 extern char letters[62]; /* complete list of letters used in smap */
@@ -257,10 +271,14 @@ extern int DIM_SIZE[BA_SYSTEM_DIMENSIONS]; /* how many midplanes in
 					    * each dimension */
 extern s_p_options_t bg_conf_file_options[]; /* used to parse the
 					      * bluegene.conf file. */
+extern uint16_t ba_deny_pass;
 
 /* Translate a state enum to a readable string */
 extern char *bg_block_state_string(rm_partition_state_t state);
 
+/* must xfree return of this */
+extern char *ba_passthroughs_string(uint16_t passthrough);
+
 /* Parse a block request from the bluegene.conf file */
 extern int parse_blockreq(void **dest, slurm_parser_enum_t type,
 			  const char *key, const char *value, 
@@ -517,4 +535,8 @@ extern int load_block_wiring(char *bg_block_id);
  */
 extern List get_and_set_block_wiring(char *bg_block_id);
 
+/* make sure a node is in the system return 1 if it is 0 if not */
+extern int validate_coord(int *coord);
+
+
 #endif /* _BLOCK_ALLOCATOR_H_ */
