@@ -41,6 +41,7 @@
 
 #include "src/common/uid.h"
 #include "src/slurmctld/trigger_mgr.h"
+#include "src/slurmctld/locks.h"
 
 /* some local functions */
 #ifdef HAVE_BG
@@ -62,7 +63,7 @@ extern void print_bg_record(bg_record_t* bg_record)
 	info("\tsize: %d BPs %u Nodes %d cpus", 
 	     bg_record->bp_count,
 	     bg_record->node_cnt,
-	     bg_record->cpus_per_bp * bg_record->bp_count);
+	     bg_record->cpu_cnt);
 	info("\tgeo: %ux%ux%u", bg_record->geo[X], bg_record->geo[Y], 
 	     bg_record->geo[Z]);
 	info("\tconn_type: %s", convert_conn_type(bg_record->conn_type));
@@ -111,25 +112,26 @@ extern void destroy_bg_record(void *object)
 	}
 }
 
+/* see if a record already of like bitmaps exists in a list */
 extern int block_exist_in_list(List my_list, bg_record_t *bg_record)
 {
 	ListIterator itr = list_iterator_create(my_list);
 	bg_record_t *found_record = NULL;
 	int rc = 0;
 
-	while ((found_record = (bg_record_t *) list_next(itr)) != NULL) {
+	while ((found_record = list_next(itr))) {
 		/* check for full node bitmap compare */
 		if(bit_equal(bg_record->bitmap, found_record->bitmap)
 		   && bit_equal(bg_record->ionode_bitmap,
 				found_record->ionode_bitmap)) {
 			if(bg_record->ionodes)
-				debug3("This block %s[%s] "
+				debug("This block %s[%s] "
 				       "is already in the list %s",
 				       bg_record->nodes,
 				       bg_record->ionodes,
 				       found_record->bg_block_id);
 			else
-				debug3("This block %s "
+				debug("This block %s "
 				       "is already in the list %s",
 				       bg_record->nodes,
 				       found_record->bg_block_id);
@@ -142,6 +144,23 @@ extern int block_exist_in_list(List my_list, bg_record_t *bg_record)
 	return rc;
 }
 
+/* see if the exact record already exists in a list */
+extern int block_ptr_exist_in_list(List my_list, bg_record_t *bg_record)
+{
+	ListIterator itr = list_iterator_create(my_list);
+	bg_record_t *found_record = NULL;
+	int rc = 0;
+
+	while ((found_record = list_next(itr))) {
+		if(bg_record == found_record) {
+			rc = 1;
+			break;
+		}
+	}
+	list_iterator_destroy(itr);
+	return rc;
+}
+
 extern void process_nodes(bg_record_t *bg_record, bool startup)
 {
 #ifdef HAVE_BG
@@ -162,7 +181,7 @@ extern void process_nodes(bg_record_t *bg_record, bool startup)
 		}
 		memset(&best_start, 0, sizeof(best_start));
 		bg_record->bp_count = 0;
-		if((bg_record->conn_type == SELECT_SMALL) && (!startup))
+		if((bg_record->conn_type >= SELECT_SMALL) && (!startup))
 			error("We shouldn't be here there could be some "
 			      "badness if we use this logic %s",
 			      bg_record->nodes);
@@ -296,12 +315,19 @@ extern void process_nodes(bg_record_t *bg_record, bool startup)
 	       alpha_num[bg_record->geo[Y]],
 	       alpha_num[bg_record->geo[Z]],
 	       bg_record->bp_count);
-
-	if ((bg_record->geo[X] == DIM_SIZE[X])
-	    && (bg_record->geo[Y] == DIM_SIZE[Y])
-	    && (bg_record->geo[Z] == DIM_SIZE[Z])) {
-		bg_record->full_block = 1;	
-	}	
+	/* This check is for sub midplane systems to figure out what
+	   the largest block can be.
+	*/
+	if((DIM_SIZE[X] > 1) || (DIM_SIZE[Y] > 1) || (DIM_SIZE[Z] > 1)) {
+		/* means we have more than 1 base partition */
+		if ((bg_record->geo[X] == DIM_SIZE[X])
+		    && (bg_record->geo[Y] == DIM_SIZE[Y])
+		    && (bg_record->geo[Z] == DIM_SIZE[Z])) {
+			bg_record->full_block = 1;	
+		}	
+	} else if(bg_record->node_cnt == bluegene_bp_node_cnt)
+		bg_record->full_block = 1;
+	
 	
 /* #ifndef HAVE_BG_FILES */
 /* 	max_dim[X] = MAX(max_dim[X], end[X]); */
@@ -420,28 +446,26 @@ extern void copy_bg_record(bg_record_t *fir_record, bg_record_t *sec_record)
 	}
 	sec_record->job_running = fir_record->job_running;
 	sec_record->job_ptr = fir_record->job_ptr;
-	sec_record->cpus_per_bp = fir_record->cpus_per_bp;
+	sec_record->cpu_cnt = fir_record->cpu_cnt;
 	sec_record->node_cnt = fir_record->node_cnt;
-#ifdef HAVE_BGL
-	sec_record->quarter = fir_record->quarter;
-	sec_record->nodecard = fir_record->nodecard;
-#endif
 }
 
 /* 
  * Comparator used for sorting blocks smallest to largest
  * 
- * returns: -1: rec_a >rec_b   0: rec_a == rec_b   1: rec_a < rec_b
+ * returns: -1: rec_a > rec_b   0: rec_a == rec_b   1: rec_a < rec_b
  * 
  */
 extern int bg_record_cmpf_inc(bg_record_t* rec_a, bg_record_t* rec_b)
 {
 	int size_a = rec_a->node_cnt;
 	int size_b = rec_b->node_cnt;
+
 	if (size_a < size_b)
 		return -1;
 	else if (size_a > size_b)
 		return 1;
+
 	if(rec_a->nodes && rec_b->nodes) {
 		size_a = strcmp(rec_a->nodes, rec_b->nodes);
 		if (size_a < 0)
@@ -449,22 +473,15 @@ extern int bg_record_cmpf_inc(bg_record_t* rec_a, bg_record_t* rec_b)
 		else if (size_a > 0)
 			return 1;
 	}
-#ifdef HAVE_BGL
-	if (rec_a->quarter < rec_b->quarter)
-		return -1;
-	else if (rec_a->quarter > rec_b->quarter)
-		return 1;
 
-	if(rec_a->nodecard < rec_b->nodecard)
-		return -1;
-	else if(rec_a->nodecard > rec_b->nodecard)
-		return 1;
-#else
+	if(!rec_a->ionode_bitmap || !rec_b->ionode_bitmap)
+		return 0;
+
 	if(bit_ffs(rec_a->ionode_bitmap) < bit_ffs(rec_b->ionode_bitmap))
 		return -1;
 	else
 		return 1;
-#endif
+
 	return 0;
 }
 
@@ -473,29 +490,25 @@ extern bg_record_t *find_bg_record_in_list(List my_list, char *bg_block_id)
 	ListIterator itr;
 	bg_record_t *bg_record = NULL;
 		
+	xassert(my_list);
+
 	if(!bg_block_id)
 		return NULL;
 			
-	if(my_list) {
-		slurm_mutex_lock(&block_state_mutex);
-		itr = list_iterator_create(my_list);
-		while ((bg_record = (bg_record_t *) list_next(itr)) != NULL) {
-			if(bg_record->bg_block_id)
-				if (!strcmp(bg_record->bg_block_id, 
-					    bg_block_id))
-					break;
-		}
-		list_iterator_destroy(itr);
-		slurm_mutex_unlock(&block_state_mutex);
-		if(bg_record)
-			return bg_record;
-		else
-			return NULL;
-	} else {
-		error("find_bg_record_in_list: no list");
-		return NULL;
+	slurm_mutex_lock(&block_state_mutex);
+	itr = list_iterator_create(my_list);
+	while ((bg_record = (bg_record_t *) list_next(itr)) != NULL) {
+		if(bg_record->bg_block_id)
+			if (!strcmp(bg_record->bg_block_id, 
+				    bg_block_id))
+				break;
 	}
-	
+	list_iterator_destroy(itr);
+	slurm_mutex_unlock(&block_state_mutex);
+	if(bg_record)
+		return bg_record;
+	else
+		return NULL;
 }
 
 /* All changes to the bg_list target_name must 
@@ -573,13 +586,30 @@ extern void drain_as_needed(bg_record_t *bg_record, char *reason)
 	bool needed = true;
 	hostlist_t hl;
 	char *host = NULL;
-	char bg_down_node[128];
 
-	if(bg_record->job_running > NO_JOB_RUNNING)
-		slurm_fail_job(bg_record->job_running);			
+	if(bg_record->job_running > NO_JOB_RUNNING) {
+		int rc;
+		slurmctld_lock_t job_write_lock = {
+			NO_LOCK, WRITE_LOCK, WRITE_LOCK, NO_LOCK };
+		lock_slurmctld(job_write_lock);
+		debug2("Trying to requeue job %d", bg_record->job_running);
+		if((rc = job_requeue(0, bg_record->job_running, -1))) {
+			error("couldn't requeue job %u, failing it: %s",
+			      bg_record->job_running, 
+			      slurm_strerror(rc));
+			job_fail(bg_record->job_running);
+		}
+		unlock_slurmctld(job_write_lock);
+		slurm_mutex_lock(&block_state_mutex);
+		if(remove_from_bg_list(bg_job_block_list, bg_record) 
+		   == SLURM_SUCCESS) {
+			num_unused_cpus += bg_record->cpu_cnt;
+		}
+		slurm_mutex_unlock(&block_state_mutex);
+	}
 
 	/* small blocks */
-	if(bg_record->cpus_per_bp != procs_per_node) {
+	if(bg_record->cpu_cnt < procs_per_node) {
 		debug2("small block");
 		goto end_it;
 	}
@@ -591,7 +621,7 @@ extern void drain_as_needed(bg_record_t *bg_record, char *reason)
 		return;
 	}
 	while ((host = hostlist_shift(hl))) {
-		if (node_already_down(bg_down_node)) {
+		if (node_already_down(host)) {
 			needed = false;
 			free(host);
 			break;
@@ -610,59 +640,27 @@ end_it:
 		sleep(1);
 	}
 	
-	slurm_mutex_lock(&block_state_mutex);
-	error("Setting Block %s to ERROR state.", bg_record->bg_block_id);
-	bg_record->job_running = BLOCK_ERROR_STATE;
-	bg_record->state = RM_PARTITION_ERROR;
-	slurm_mutex_unlock(&block_state_mutex);
-	trigger_block_error();
+	put_block_in_error_state(bg_record, BLOCK_ERROR_STATE);
 	return;
 }
 
-#ifdef HAVE_BGL
-
-extern int set_ionodes(bg_record_t *bg_record)
+extern int set_ionodes(bg_record_t *bg_record, int io_start, int io_nodes)
 {
-	int i = 0;
-	int start_bit = 0;
-	int size = 0;
 	char bitstring[BITSIZE];
-	
+
 	if(!bg_record)
 		return SLURM_ERROR;
-	/* set the bitmap blank here if it is a full node we don't
-	   want anything set we also don't want the bg_record->ionodes set.
-	*/
+	
 	bg_record->ionode_bitmap = bit_alloc(bluegene_numpsets);
-	if(bg_record->quarter == (uint16_t)NO_VAL) {
-		return SLURM_SUCCESS;
-	}
-
-	start_bit = bluegene_quarter_ionode_cnt*bg_record->quarter;
-	
-	if(bg_record->nodecard != (uint16_t)NO_VAL
-	   && bluegene_nodecard_ionode_cnt) {
-		start_bit += bluegene_nodecard_ionode_cnt*bg_record->nodecard;
-		size = bluegene_nodecard_ionode_cnt;
-	} else
-		size = bluegene_quarter_ionode_cnt;
-	size += start_bit;
-
-	if(size == start_bit) {
-		error("start bit is the same as the end bit %d", size);
-		return SLURM_ERROR;
-	}
-	for(i=start_bit; i<size; i++)
-		bit_set(bg_record->ionode_bitmap, i);
-	
+	/* Set the correct ionodes being used in this block */
+	bit_nset(bg_record->ionode_bitmap, io_start, io_start+io_nodes);
 	bit_fmt(bitstring, BITSIZE, bg_record->ionode_bitmap);
 	bg_record->ionodes = xstrdup(bitstring);
-
 	return SLURM_SUCCESS;
 }
-#endif
 
-extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
+extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq,
+			 bool no_check, bitoff_t io_start)
 {
 	bg_record_t *bg_record = NULL;
 	ba_node_t *ba_node = NULL;
@@ -670,23 +668,15 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 	uid_t pw_uid;
 	int i, len;
 	int small_count = 0;
-#ifdef HAVE_BGL
-	int node_cnt = 0;
-	uint16_t quarter = 0;
-	uint16_t nodecard = 0;
-	int small_size = 0;
-	bg_record_t *found_record = NULL;
-#endif
+
 	if(!records) {
 		fatal("add_bg_record: no records list given");
 	}
 	bg_record = (bg_record_t*) xmalloc(sizeof(bg_record_t));
 	
 	
-	bg_record->user_name = 
-		xstrdup(bg_slurm_user_name);
-	bg_record->target_name = 
-		xstrdup(bg_slurm_user_name);
+	bg_record->user_name = xstrdup(bg_slurm_user_name);
+	bg_record->target_name = xstrdup(bg_slurm_user_name);
 	
 	pw_uid = uid_from_string(bg_record->user_name);
 	if(pw_uid == (uid_t) -1) {
@@ -705,10 +695,8 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 	/* bg_record->boot_state = 0; 	Implicit */
 	/* bg_record->state = 0;	Implicit */
 #ifdef HAVE_BGL
-	bg_record->quarter = (uint16_t)NO_VAL;
-	bg_record->nodecard = (uint16_t)NO_VAL;
 	debug2("asking for %s %d %d %s", 
-	       blockreq->block, blockreq->small128, blockreq->small32,
+	       blockreq->block, blockreq->small32, blockreq->small128,
 	       convert_conn_type(blockreq->conn_type));
 #else
 	debug2("asking for %s %d %d %d %d %d %s", 
@@ -746,7 +734,7 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 	bg_record->node_use = SELECT_COPROCESSOR_MODE;
 #endif
 	bg_record->conn_type = blockreq->conn_type;
-	bg_record->cpus_per_bp = procs_per_node;
+	bg_record->cpu_cnt = procs_per_node * bg_record->bp_count;
 	bg_record->node_cnt = bluegene_bp_node_cnt * bg_record->bp_count;
 	bg_record->job_running = NO_JOB_RUNNING;
 
@@ -785,79 +773,8 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 		}
 	} else {
 		debug("adding a small block");
-#ifdef HAVE_BGL // remove this clause when other works.  Only here to
-		// perserve old code 
-
-		/* if the ionode cnt for small32 is 0 then don't
-		   allow a nodecard allocation 
-		*/
-		if(!bluegene_nodecard_ionode_cnt) {
-			if(blockreq->small32) 
-				fatal("There is an error in your "
-				      "bluegene.conf file.\n"
-				      "Can't create a 32 node block with "
-				      "Numpsets=%u. (Try setting it to 64)",
-				      bluegene_numpsets);
-		}
-
-		if(blockreq->small32==0 && blockreq->small128==0) {
-			info("No specs given for this small block, "
-			     "I am spliting this block into 4 128CnBlocks");
-			blockreq->small128=4;
-		}		
-
-		i = (blockreq->small32*bluegene_nodecard_node_cnt) + 
-			(blockreq->small128*bluegene_quarter_node_cnt);
-		if(i != bluegene_bp_node_cnt)
-			fatal("There is an error in your bluegene.conf file.\n"
-			      "I am unable to request %d nodes consisting of "
-			      "%u 32CnBlocks and\n%u 128CnBlocks in one "
-			      "base partition with %u nodes.", 
-			      i, bluegene_bp_node_cnt, 
-			      blockreq->small32, blockreq->small128);
-		small_count = blockreq->small32+blockreq->small128; 
-		/* Automatically create 4-way split if 
-		 * conn_type == SELECT_SMALL in bluegene.conf
-		 * Here we go through each node listed and do the same thing
-		 * for each node.
-		 */
-		itr = list_iterator_create(bg_record->bg_block_list);
-		while ((ba_node = list_next(itr)) != NULL) {
-			/* break base partition up into 16 parts */
-			small_size = bluegene_bp_nodecard_cnt;
-			node_cnt = 0;
-			quarter = 0;
-			nodecard = 0;
-			for(i=0; i<small_count; i++) {
-				if(i == blockreq->small32) {
-					/* break base partition 
-					   up into 4 parts */
-					small_size = 4;
-				}
-									
-				if(small_size == 4)
-					nodecard = (uint16_t)NO_VAL;
-				else
-					nodecard = i%4; 
-				found_record = create_small_record(bg_record,
-								   quarter,
-								   nodecard);
-								 
-				/* this needs to be an append so we
-				   keep things in the order we got
-				   them, they will be sorted later */
-				list_append(records, found_record);
-				node_cnt += bluegene_bp_node_cnt/small_size;
-				if(node_cnt == 128) {
-					node_cnt = 0;
-					quarter++;
-				}
-			}
-		}
-		list_iterator_destroy(itr);
-		destroy_bg_record(bg_record);
-#else // remove this when testing.  Only here to perserve old code the
-      // code below is already for bgl
+		if(no_check)
+			goto no_check;
 		/* if the ionode cnt for small32 is 0 then don't
 		   allow a sub quarter allocation 
 		*/
@@ -934,6 +851,7 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 			+ blockreq->small128
 			+ blockreq->small256; 
 #endif
+	no_check:
 		/* Automatically create 2-way split if 
 		 * conn_type == SELECT_SMALL in bluegene.conf
 		 * Here we go through each node listed and do the same thing
@@ -942,17 +860,15 @@ extern int add_bg_record(List records, List used_nodes, blockreq_t *blockreq)
 		itr = list_iterator_create(bg_record->bg_block_list);
 		while ((ba_node = list_next(itr)) != NULL) {
 			handle_small_record_request(records, blockreq,
-						    bg_record, 0);
+						    bg_record, io_start);
 		}
 		list_iterator_destroy(itr);
 		destroy_bg_record(bg_record);
-#endif // remove this when done testing
 	} 
 	
 	return SLURM_SUCCESS;
 }
 
-#ifndef HAVE_BGL
 extern int handle_small_record_request(List records, blockreq_t *blockreq,
 				       bg_record_t *bg_record, bitoff_t start)
 {
@@ -964,6 +880,9 @@ extern int handle_small_record_request(List records, blockreq_t *blockreq,
 	xassert(blockreq);
 	xassert(bg_record);
 
+	xassert(start >= 0);
+	xassert(start < bluegene_numpsets);
+
 #ifndef HAVE_BGL
 	for(i=0; i<blockreq->small16; i++) {
 		bit_nset(ionodes, start, start);
@@ -1036,7 +955,6 @@ extern int handle_small_record_request(List records, blockreq_t *blockreq,
 
 	return SLURM_SUCCESS;
 }
-#endif
 
 extern int format_node_name(bg_record_t *bg_record, char *buf, int buf_size)
 {
@@ -1050,6 +968,421 @@ extern int format_node_name(bg_record_t *bg_record, char *buf, int buf_size)
 	return SLURM_SUCCESS;
 }
 
+extern int down_nodecard(char *bp_name, bitoff_t io_start)
+{
+	List requests = NULL;
+	List delete_list = NULL;
+	ListIterator itr = NULL;
+	bg_record_t *bg_record = NULL, *found_record = NULL, tmp_record;
+	bg_record_t *smallest_bg_record = NULL;
+	struct node_record *node_ptr = NULL;
+	int bp_bit = 0;
+	static int io_cnt = NO_VAL;
+	static int create_size = NO_VAL;
+	static blockreq_t blockreq; 
+	int rc = SLURM_SUCCESS;
+
+	xassert(bp_name);
+
+	if(io_cnt == NO_VAL) {
+		io_cnt = 1;
+		/* Translate 1 nodecard count to ionode count */
+		if((io_cnt *= bluegene_io_ratio))
+			io_cnt--;
+		/* make sure we create something that is able to be
+		   created */
+		if(bluegene_smallest_block < bluegene_nodecard_node_cnt)
+			create_size = bluegene_nodecard_node_cnt;
+		else
+			create_size = bluegene_smallest_block;
+	}
+
+	node_ptr = find_node_record(bp_name);
+	if (!node_ptr) {
+		error ("down_sub_node_blocks: invalid node specified '%s'",
+		       bp_name);
+		return EINVAL;
+	}
+	bp_bit = (node_ptr - node_record_table_ptr);
+	
+	memset(&blockreq, 0, sizeof(blockreq_t));
+	
+	blockreq.conn_type = SELECT_SMALL;
+	blockreq.block = bp_name;
+
+	debug3("here setting %d of %d and %d-%d of %d",
+	       bp_bit, node_record_count, io_start, 
+	       io_start+io_cnt, bluegene_numpsets);
+
+	memset(&tmp_record, 0, sizeof(bg_record_t));
+	tmp_record.bp_count = 1;
+	tmp_record.node_cnt = bluegene_nodecard_node_cnt;
+	tmp_record.bitmap = bit_alloc(node_record_count);
+	bit_set(tmp_record.bitmap, bp_bit);
+
+	tmp_record.ionode_bitmap = bit_alloc(bluegene_numpsets);
+	bit_nset(tmp_record.ionode_bitmap, io_start, io_start+io_cnt);
+
+	slurm_mutex_lock(&block_state_mutex);
+	itr = list_iterator_create(bg_list);
+	while ((bg_record = list_next(itr))) {
+		if(!bit_test(bg_record->bitmap, bp_bit))
+			continue;
+		
+		if(!blocks_overlap(bg_record, &tmp_record)) 
+			continue;
+
+		if(bg_record->job_running > NO_JOB_RUNNING) 
+			slurm_fail_job(bg_record->job_running);
+
+		/* mark every one of these in an error state */
+		if(bluegene_layout_mode != LAYOUT_DYNAMIC) {
+			if(!delete_list)
+				delete_list = list_create(NULL);
+			list_append(delete_list, bg_record);
+			continue;
+		} 
+
+		/* below is only for dynamic modes since there are
+		   never overlapping blocks there */
+		/* if the block is smaller than the create size just
+		   continue on.
+		*/
+		if(bg_record->node_cnt < create_size)
+			continue;
+
+		if(!smallest_bg_record || 
+		   (smallest_bg_record->node_cnt > bg_record->node_cnt))
+			smallest_bg_record = bg_record;
+	}
+	list_iterator_destroy(itr);
+	slurm_mutex_unlock(&block_state_mutex);
+	
+	if(bluegene_layout_mode != LAYOUT_DYNAMIC) {
+		debug3("running non-dynamic mode");
+		if(delete_list) {
+			int cnt_set = 0;
+			/* don't lock here since it is handled inside
+			   the put_block_in_error_state
+			*/
+			itr = list_iterator_create(delete_list);
+			while ((bg_record = list_next(itr))) {
+				/* we already handled this */
+				if(bg_record->state == RM_PARTITION_ERROR) {
+					rc = SLURM_NO_CHANGE_IN_DATA;
+					continue;
+				}
+								
+				rc = put_block_in_error_state(
+					bg_record, BLOCK_ERROR_STATE);
+				cnt_set++;
+			}
+			if(cnt_set)
+				rc = SLURM_SUCCESS;
+			list_iterator_destroy(itr);
+			list_destroy(delete_list);
+			goto cleanup;
+		} 
+		
+		debug("didn't get a smallest block");
+		if(!node_already_down(bp_name)) {
+			time_t now = time(NULL);
+			char reason[128], time_str[32];
+			slurm_make_time_str(&now, time_str,
+					    sizeof(time_str));
+			snprintf(reason, sizeof(reason), 
+				 "select_bluegene: "
+				 "nodecard down [SLURM@%s]", 
+				 time_str); 
+			slurm_drain_nodes(bp_name, reason);
+		}
+		rc = SLURM_SUCCESS;
+		goto cleanup;
+	} 
+
+	
+	if(smallest_bg_record) {
+		debug2("smallest dynamic block is %s",
+		       smallest_bg_record->bg_block_id);
+		if(smallest_bg_record->state == RM_PARTITION_ERROR) {
+			rc = SLURM_NO_CHANGE_IN_DATA;
+			goto cleanup;
+		}
+		
+		while(smallest_bg_record->job_running > NO_JOB_RUNNING)
+			sleep(1);
+
+		if(smallest_bg_record->node_cnt == create_size) {
+			rc = put_block_in_error_state(
+				smallest_bg_record, BLOCK_ERROR_STATE);
+			goto cleanup;
+		} 
+
+		if(create_size > smallest_bg_record->node_cnt) {
+			/* we should never get here.  This means we
+			 * have a create_size that is bigger than a
+			 * block that is already made.
+			 */
+			rc = put_block_in_error_state(
+				smallest_bg_record, BLOCK_ERROR_STATE);
+			goto cleanup;
+		}
+		debug3("node count is %d", smallest_bg_record->node_cnt);
+		switch(smallest_bg_record->node_cnt) {
+#ifndef HAVE_BGL
+		case 64:
+			blockreq.small32 = 2;
+			break;
+		case 256:
+			blockreq.small32 = 8;
+			break;
+#endif
+		case 128:
+			blockreq.small32 = 4;			
+			break;
+		case 512:
+		default:
+			blockreq.small32 = 16;
+			break;
+		}
+
+		if(create_size != bluegene_nodecard_node_cnt) {
+			blockreq.small128 = blockreq.small32 / 4;
+			blockreq.small32 = 0;
+		}
+		/* set the start to be the same as the start of the
+		   ionode_bitmap.  If no ionodes set (not a small
+		   block) set io_start = 0. */
+		if((io_start = bit_ffs(smallest_bg_record->ionode_bitmap))
+		   == -1)
+			io_start = 0;
+	} else {
+		switch(create_size) {
+#ifndef HAVE_BGL
+		case 64:
+			blockreq.small64 = 8;
+			break;
+		case 256:
+			blockreq.small256 = 2;
+#endif
+		case 32:
+			blockreq.small32 = 16;
+			break;
+		case 128:
+			blockreq.small128 = 4;
+			break;
+		case 512:
+			if(!node_already_down(bp_name)) {
+				time_t now = time(NULL);
+				char reason[128], time_str[32];
+				slurm_make_time_str(&now, time_str,
+						    sizeof(time_str));
+				snprintf(reason, sizeof(reason), 
+					 "select_bluegene: "
+					 "nodecard down [SLURM@%s]", 
+					 time_str); 
+				slurm_drain_nodes(bp_name, reason);
+			}
+			rc = SLURM_SUCCESS;
+			goto cleanup;
+			break;
+		default:
+			error("Unknown create size of %d", create_size);
+			break;
+		}
+		/* since we don't have a block in this midplane
+		   we need to start at the beginning. */
+		io_start = 0;
+		/* we also need a bg_block to pretend to be the
+		   smallest block that takes up the entire midplane. */
+	}
+		
+	
+	/* Here we need to add blocks that take up nodecards on this
+	   midplane.  Since Slurm only keeps track of midplanes
+	   natively this is the only want to handle this case.
+	*/
+	requests = list_create(destroy_bg_record);
+	add_bg_record(requests, NULL, &blockreq, 1, io_start);
+		
+	delete_list = list_create(NULL);
+	while((bg_record = list_pop(requests))) {
+		slurm_mutex_lock(&block_state_mutex);
+		itr = list_iterator_create(bg_list);
+		while((found_record = list_next(itr))) {
+			if(!blocks_overlap(bg_record, found_record))
+				continue;
+			list_push(delete_list, found_record);
+			list_remove(itr);
+			num_block_to_free++;
+		}
+		list_iterator_destroy(itr);
+		slurm_mutex_unlock(&block_state_mutex);
+
+		/* we need to add this record since it doesn't exist */
+		if(configure_block(bg_record) == SLURM_ERROR) {
+			destroy_bg_record(bg_record);
+			error("down_sub_node_blocks: "
+			      "unable to configure block in api");
+			continue;
+		}
+
+		debug("adding block %s to fill in small blocks "
+		      "around bad nodecards",
+		      bg_record->bg_block_id);
+		print_bg_record(bg_record);
+		slurm_mutex_lock(&block_state_mutex);
+		list_append(bg_list, bg_record);
+		slurm_mutex_unlock(&block_state_mutex);
+		if(bit_overlap(bg_record->ionode_bitmap, 
+			       tmp_record.ionode_bitmap)) {
+			/* here we know the error block doesn't exist
+			   so just set the state here */
+			rc = put_block_in_error_state(
+				bg_record, BLOCK_ERROR_STATE);
+		}
+	}
+	list_destroy(requests);
+	
+	slurm_mutex_lock(&block_state_mutex);
+	free_block_list(delete_list);
+	list_destroy(delete_list);
+	sort_bg_record_inc_size(bg_list);
+	slurm_mutex_unlock(&block_state_mutex);
+	last_bg_update = time(NULL);	
+
+cleanup:
+	FREE_NULL_BITMAP(tmp_record.bitmap);
+	FREE_NULL_BITMAP(tmp_record.ionode_bitmap);
+
+	return rc;
+	
+}
+
+extern int up_nodecard(char *bp_name, bitstr_t *ionode_bitmap)
+{
+	ListIterator itr = NULL;
+	bg_record_t *bg_record = NULL;
+	struct node_record *node_ptr = NULL;
+	int bp_bit = 0;
+	int ret = 0;
+
+	xassert(bp_name);
+	xassert(ionode_bitmap);
+
+	node_ptr = find_node_record(bp_name);
+	if (!node_ptr) {
+		error ("down_sub_node_blocks: invalid node specified %s",
+		       bp_name);
+		return EINVAL;
+	}
+	bp_bit = (node_ptr - node_record_table_ptr);
+	
+	slurm_mutex_lock(&block_state_mutex);
+	itr = list_iterator_create(bg_list);
+	while((bg_record = list_next(itr))) {
+		if(bg_record->job_running != BLOCK_ERROR_STATE)
+			continue;
+		if(!bit_test(bg_record->bitmap, bp_bit))
+			continue;
+		
+		if(!bit_overlap(bg_record->ionode_bitmap, ionode_bitmap)) {
+			continue;
+		}
+		resume_block(bg_record);			
+	}
+	list_iterator_destroy(itr);
+	slurm_mutex_unlock(&block_state_mutex);
+	
+	/* FIX ME: This needs to call the opposite of
+	   slurm_drain_nodes which does not yet exist.
+	*/
+	if((ret = node_already_down(bp_name))) {
+		/* means it was drained */
+		if(ret == 2) {
+			/* debug("node %s put back into service after " */
+/* 			      "being in an error state", */
+/* 			      bp_name); */
+		}
+	}
+
+	return SLURM_SUCCESS;
+}
+
+extern int put_block_in_error_state(bg_record_t *bg_record, int state)
+{
+	uid_t pw_uid;
+
+	xassert(bg_record);
+	
+	/* only check this if the blocks are created, meaning this
+	   isn't at startup.
+	*/
+	if(blocks_are_created) {
+		/* Since we are putting this block in an error state we need
+		   to wait for the job to be removed.  We don't really
+		   need to free the block though since we may just
+		   want it to be in an error state for some reason. */
+		while(bg_record->job_running > NO_JOB_RUNNING)
+			sleep(1);
+	}
+	
+	info("Setting Block %s to ERROR state.", bg_record->bg_block_id);
+	/* we add the block to these lists so we don't try to schedule
+	   on them. */
+	if(!block_ptr_exist_in_list(bg_job_block_list, bg_record)) {
+		list_push(bg_job_block_list, bg_record);
+		num_unused_cpus -= bg_record->cpu_cnt;
+	}
+	if(!block_ptr_exist_in_list(bg_booted_block_list, bg_record)) 
+		list_push(bg_booted_block_list, bg_record);
+	
+	slurm_mutex_lock(&block_state_mutex);
+	bg_record->job_running = state;
+	bg_record->state = RM_PARTITION_ERROR;
+
+	xfree(bg_record->user_name);
+	xfree(bg_record->target_name);
+	bg_record->user_name = xstrdup(bg_slurm_user_name);
+	bg_record->target_name = xstrdup(bg_slurm_user_name);
+	
+	pw_uid = uid_from_string(bg_record->user_name);
+	if(pw_uid == (uid_t) -1) {
+		error("No such user: %s", bg_record->user_name);
+	} else {
+		bg_record->user_uid = pw_uid;
+	}
+	slurm_mutex_unlock(&block_state_mutex);
+
+	trigger_block_error();
+	last_bg_update = time(NULL);
+
+	return SLURM_SUCCESS;
+}
+
+/* block_state_mutex should be locked before calling */
+extern int resume_block(bg_record_t *bg_record)
+{
+	xassert(bg_record);
+
+	if(bg_record->job_running >= NO_JOB_RUNNING)
+		return SLURM_SUCCESS;
+
+	info("Block %s put back into service after "
+	     "being in an error state.",
+	      bg_record->bg_block_id);
+
+	if(remove_from_bg_list(bg_job_block_list, bg_record) == SLURM_SUCCESS) 
+		num_unused_cpus += bg_record->cpu_cnt;
+	remove_from_bg_list(bg_booted_block_list, bg_record);
+
+	bg_record->job_running = NO_JOB_RUNNING;
+	bg_record->state = RM_PARTITION_FREE;
+	last_bg_update = time(NULL);
+
+	return SLURM_SUCCESS;
+}
+
 /************************* local functions ***************************/
 
 #ifdef HAVE_BG
@@ -1125,6 +1458,8 @@ static int _ba_node_cmpf_inc(ba_node_t *node_a, ba_node_t *node_b)
 	      alpha_num[node_a->coord[Z]]); 
 	return 0;
 }
+
+
 #endif //HAVE_BG
 
 
