@@ -42,9 +42,10 @@
 #include "bluegene.h"
 
 extern List create_dynamic_block(List block_list,
-				 ba_request_t *request, List my_block_list);
+				 ba_request_t *request, List my_block_list,
+				 bool track_down_nodes);
 
-#ifdef HAVE_BGL 
+#ifdef HAVE_BGQ 
 extern bg_record_t *create_small_record(bg_record_t *bg_record, 
 					uint16_t quarter, uint16_t nodecard);
 #else
