@@ -661,20 +661,34 @@ static int _find_job_mate(struct job_record *job_ptr, bitstr_t *bitmap,
 {
 	ListIterator job_iterator;
 	struct job_record *job_scan_ptr;
+	int rc = EINVAL;
 
 	job_iterator = list_iterator_create(job_list);
 	while ((job_scan_ptr = (struct job_record *) list_next(job_iterator))) {
-		if ((job_scan_ptr->part_ptr == job_ptr->part_ptr) &&
-		    (job_scan_ptr->job_state == JOB_RUNNING) &&
-		    (job_scan_ptr->node_cnt == req_nodes) &&
-		    (job_scan_ptr->total_procs >= job_ptr->num_procs) &&
-		    bit_super_set(job_scan_ptr->node_bitmap, bitmap)) {
-			bit_and(bitmap, job_scan_ptr->node_bitmap);
-			return SLURM_SUCCESS;
-		}
+		if ((job_scan_ptr->part_ptr  != job_ptr->part_ptr) ||
+		    (job_scan_ptr->job_state != JOB_RUNNING) ||
+		    (job_scan_ptr->node_cnt  != req_nodes) ||
+		    (job_scan_ptr->total_procs < job_ptr->num_procs) ||
+		    (!bit_super_set(job_scan_ptr->node_bitmap, bitmap)))
+			continue;
+
+		if (job_ptr->details->req_node_bitmap &&
+		    (!bit_super_set(job_ptr->details->req_node_bitmap,
+				    job_scan_ptr->node_bitmap)))
+			continue;	/* Required nodes missing from job */
+
+		if (job_ptr->details->exc_node_bitmap &&
+		    (bit_overlap(job_ptr->details->exc_node_bitmap,
+				 job_scan_ptr->node_bitmap) != 0))
+			continue;	/* Excluded nodes in this job */
+
+		bit_and(bitmap, job_scan_ptr->node_bitmap);
+		job_ptr->total_procs = job_scan_ptr->total_procs;
+		rc = SLURM_SUCCESS;
+		break;
 	}
 	list_iterator_destroy(job_iterator);
-	return EINVAL;
+	return rc;
 }
 
 /* _job_test - does most of the real work for select_p_job_test(), which 
