@@ -86,7 +86,7 @@ int _sacct_query(slurm_step_layout_t *step_layout, uint32_t job_id,
 	msg.data            = &r;
 	
 	
-	ret_list = slurm_send_recv_msgs(step_layout->node_list, &msg, 0);
+	ret_list = slurm_send_recv_msgs(step_layout->node_list, &msg, 0, false);
 	if (!ret_list) {
 		error("got an error no list returned");
 		goto cleanup;
