@@ -101,7 +101,9 @@ static int _set_cond(int *start, int argc, char *argv[],
 			  || !strncasecmp (argv[i], "Names", 
 					   MAX(command_len, 1))
 			  || !strncasecmp (argv[i], "Accounts",
-					   MAX(command_len, 1))) {
+					   MAX(command_len, 1))
+			  || !strncasecmp (argv[i], "Acct",
+					   MAX(command_len, 4))) {
 			if(!assoc_cond->acct_list) {
 				assoc_cond->acct_list = 
 					list_create(slurm_destroy_char);
@@ -134,6 +136,8 @@ static int _set_cond(int *start, int argc, char *argv[],
 			if(format_list)
 				slurm_addto_char_list(format_list, argv[i]+end);
 		} else if (!strncasecmp (argv[i], "FairShare", 
+					 MAX(command_len, 1))
+			   || !strncasecmp (argv[i], "Shares",
 					 MAX(command_len, 1))) {
 			if(!assoc_cond->fairshare_list)
 				assoc_cond->fairshare_list =
@@ -339,7 +343,9 @@ static int _set_rec(int *start, int argc, char *argv[],
 			  || !strncasecmp (argv[i], "Account",
 					   MAX(command_len, 1))
 			  || !strncasecmp (argv[i], "Names", 
-					   MAX(command_len, 1))) {
+					   MAX(command_len, 1))
+			  || !strncasecmp (argv[i], "Acct",
+					    MAX(command_len, 4))) {
 			if(acct_list) 
 				slurm_addto_char_list(acct_list, argv[i]+end);
 				
@@ -353,6 +359,8 @@ static int _set_rec(int *start, int argc, char *argv[],
 			acct->description =  strip_quotes(argv[i]+end, NULL, 1);
 			u_set = 1;
 		} else if (!strncasecmp (argv[i], "FairShare", 
+					 MAX(command_len, 1))
+			   || !strncasecmp (argv[i], "Shares",
 					 MAX(command_len, 1))) {
 			if(!assoc)
 				continue;
@@ -1123,6 +1131,12 @@ extern int sacctmgr_list_account(int argc, char *argv[])
 			field->name = xstrdup("Par Name");
 			field->len = 10;
 			field->print_routine = print_fields_str;
+		} else if(!strncasecmp("Shares", object,
+				       MAX(command_len, 1))) {
+			field->type = PRINT_FAIRSHARE;
+			field->name = xstrdup("Shares");
+			field->len = 9;
+			field->print_routine = print_fields_uint;
 		} else if(!strncasecmp("User", object, MAX(command_len, 1))) {
 			field->type = PRINT_USER;
 			field->name = xstrdup("User");
