@@ -108,7 +108,9 @@ static int _set_cond(int *start, int argc, char *argv[],
 			list_iterator_destroy(itr);
 			set = 1;
 		} else if (!strncasecmp (argv[i], "Accounts",
-					 MAX(command_len, 2))) {
+					 MAX(command_len, 2))
+			   || !strncasecmp (argv[i], "Acct",
+					    MAX(command_len, 4))) {
 			if(!assoc_cond->acct_list)
 				assoc_cond->acct_list = 
 					list_create(slurm_destroy_char);
@@ -129,6 +131,8 @@ static int _set_cond(int *start, int argc, char *argv[],
 				slurm_addto_char_list(format_list,
 						      argv[i]+end);
 		} else if (!strncasecmp (argv[i], "FairShare",
+					 MAX(command_len, 1))
+			   || !strncasecmp (argv[i], "Shares",
 					 MAX(command_len, 1))) {
 			if(!assoc_cond->fairshare_list)
 				assoc_cond->fairshare_list =
@@ -373,7 +377,8 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 	
 		field = xmalloc(sizeof(print_field_t));
 
-		if(!strncasecmp("Account", object, MAX(command_len, 1))) {
+		if(!strncasecmp("Account", object, MAX(command_len, 1))
+		   || !strncasecmp("Acct", object, MAX(command_len, 4))) {
 			field->type = PRINT_ACCOUNT;
 			field->name = xstrdup("Account");
 			if(tree_display)
@@ -522,6 +527,12 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->name = xstrdup("RGT");
 			field->len = 6;
 			field->print_routine = print_fields_uint;
+		} else if(!strncasecmp("Shares", object,
+				       MAX(command_len, 1))) {
+			field->type = PRINT_FAIRSHARE;
+			field->name = xstrdup("Shares");
+			field->len = 9;
+			field->print_routine = print_fields_uint;
 		} else if(!strncasecmp("User", object, MAX(command_len, 1))) {
 			field->type = PRINT_USER;
 			field->name = xstrdup("User");
