@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  sbatch.c - Submit a SLURM batch script.
  *
- *  $Id: sbatch.c 14469 2008-07-09 18:15:23Z jette $
+ *  $Id: sbatch.c 15808 2008-12-02 23:38:47Z da $
  *****************************************************************************
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Copyright (C) 2008 Lawrence Livermore National Security.
@@ -42,7 +42,6 @@
 #include <slurm/slurm.h>
 
 #include "src/common/env.h"
-#include "src/common/plugstack.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/xstring.h"
@@ -52,8 +51,6 @@
 
 #define MAX_RETRIES 3
 
-static void _call_spank_local_user(job_desc_msg_t desc,
-				   submit_response_msg_t *resp);
 static int   fill_job_desc_from_opts(job_desc_msg_t *desc);
 static void *get_script_buffer(const char *filename, int *size);
 static void  set_prio_process_env(void);
@@ -72,9 +69,6 @@ int main(int argc, char *argv[])
 	int retries = 0;
 
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
-	if (spank_init(NULL) < 0)
-		fatal("Plug-in initialization failed");
-
 	script_name = process_options_first_pass(argc, argv);
 	/* reinit log with new verbosity (if changed by command line) */
 	if (opt.verbose || opt.quiet) {
@@ -97,7 +91,13 @@ int main(int argc, char *argv[])
 		fatal("sbatch parameter parsing");
 	}
 
-	(void) _set_rlimit_env();
+	if (opt.get_user_env_time < 0) {
+		/* Moab does not propage the user's resource limits, so 
+		 * slurmd determines the values at the same time that it 
+		 * gets the user's default environment variables. */
+		(void) _set_rlimit_env();
+	}
+
 	set_prio_process_env();
 	set_umask_env();
 	slurm_init_job_desc_msg(&desc);
@@ -122,31 +122,12 @@ int main(int argc, char *argv[])
 			error(msg);
 		sleep (++retries);
         }
-	_call_spank_local_user(desc, resp);
 	info("Submitted batch job %d", resp->job_id);
 	xfree(desc.script);
 	slurm_free_submit_response_response_msg(resp);
-	spank_fini(NULL);
 	return 0;
 }
 
-static void _call_spank_local_user(job_desc_msg_t desc,
-				   submit_response_msg_t *resp)
-{
-	struct spank_launcher_job_info info[1];
-
-	info->uid = desc.user_id;
-	info->gid = desc.group_id;
-	info->jobid = resp->job_id;
-	info->stepid = SLURM_BATCH_SCRIPT;
-	info->step_layout = NULL;
-	info->argc = desc.argc;
-	info->argv = desc.argv;
-
-	if (spank_local_user(info) < 0)
-		error("spank_local_user: %m");
-}
-
 /* Returns 0 on success, -1 on failure */
 static int fill_job_desc_from_opts(job_desc_msg_t *desc)
 {
@@ -158,9 +139,13 @@ static int fill_job_desc_from_opts(job_desc_msg_t *desc)
 	desc->features = opt.constraints;
 	desc->immediate = opt.immediate;
 	if (opt.job_name != NULL)
-		desc->name = opt.job_name;
+		desc->name = xstrdup(opt.job_name);
 	else
 		desc->name = xstrdup("sbatch");
+
+	if(opt.wckey)
+ 		xstrfmtcat(desc->name, "\"%s", opt.wckey);
+
 	desc->req_nodes = opt.nodelist;
 	desc->exc_nodes = opt.exc_nodes;
 	desc->partition = opt.partition;
@@ -371,6 +356,24 @@ static bool contains_null_char(const void *buf, int size)
 	return false;
 }
 
+/*
+ * Checks if the buffer contains any DOS linebreak (\r\n).
+ */
+static bool contains_dos_linebreak(const void *buf, int size)
+{
+	char *str = (char *)buf;
+	char prev_char = '\0';
+	int i;
+
+	for (i = 0; i < size; i++) {
+		if (prev_char == '\r' && str[i] == '\n')
+			return true;
+		prev_char = str[i];
+	}
+
+	return false;
+}
+
 /*
  * If "filename" is NULL, the batch script is read from standard input.
  */
@@ -413,7 +416,8 @@ static void *get_script_buffer(const char *filename, int *size)
 		ptr = buf + script_size;
 		buf_left = buf_size - script_size;
 	}
-	close(fd);
+	if (filename)
+		close(fd);
 
 	/*
 	 * Finally we perform some sanity tests on the script.
@@ -421,9 +425,6 @@ static void *get_script_buffer(const char *filename, int *size)
 	if (script_size == 0) {
 		error("Batch script is empty!");
 		goto fail;
-	} else if (script_size >= 0xffff) {
-		error("Job script exceeds size supported by slurm");
-		goto fail;
 	} else if (xstring_is_whitespace(buf)) {
 		error("Batch script contains only whitespace!");
 		goto fail;
@@ -437,6 +438,10 @@ static void *get_script_buffer(const char *filename, int *size)
 		error("The SLURM controller does not allow scripts that");
 		error("contain a NULL character '\\0'.");
 		goto fail;
+	} else if (contains_dos_linebreak(buf, script_size)) {
+		error("Batch script contains DOS line breaks (\\r\\n)");
+		error("instead of expected UNIX line breaks (\\n).");
+		goto fail;
 	}
 
 	*size = script_size;
@@ -474,8 +479,15 @@ static int _set_rlimit_env(void)
 	slurm_conf_lock();
 	slurm_conf_unlock();
 
+	/* Modify limits with any command-line options */
+	if (opt.propagate && parse_rlimits( opt.propagate, PROPAGATE_RLIMITS))
+		fatal( "--propagate=%s is not valid.", opt.propagate );
+
 	for (rli = get_slurm_rlimits_info(); rli->name != NULL; rli++ ) {
 
+		if (rli->propagate_flag != PROPAGATE_RLIMITS)
+			continue;
+
 		if (getrlimit (rli->resource, rlim) < 0) {
 			error ("getrlimit (RLIMIT_%s): %m", rli->name);
 			rc = SLURM_FAILURE;
@@ -510,7 +522,7 @@ static int _set_rlimit_env(void)
 	if (rlim->rlim_cur < rlim->rlim_max) {
 		rlim->rlim_cur = rlim->rlim_max;
 		if (setrlimit (RLIMIT_NOFILE, rlim) < 0) 
-			return (error ("Unable to increase max no. files: %m"));
+			return (error("Unable to increase max no. files: %m"));
 	}
 
 	return rc;
