@@ -258,6 +258,7 @@ scontrol_update_job (int argc, char *argv[])
 						(char **) NULL, 10);
 			update_cnt++;
 		}
+		/* MinNodes was replaced by ReqNodes in SLURM version 1.2 */
 		else if ((strncasecmp(argv[i], "MinNodes=", 9) == 0) ||
 		         (strncasecmp(argv[i], "ReqNodes=", 9) == 0)) {
 			char *tmp;
@@ -452,7 +453,7 @@ scontrol_update_job (int argc, char *argv[])
 			update_cnt++;
 		}
 		else if (strncasecmp(argv[i], "StartTime=", 10) == 0) {
-			job_msg.begin_time = parse_time(&argv[i][10]);
+			job_msg.begin_time = parse_time(&argv[i][10], 0);
 			update_cnt++;
 		}
 		else {
