@@ -101,7 +101,7 @@ extern void acct_policy_add_job_submit(struct job_record *job_ptr)
 {
 	acct_association_rec_t *assoc_ptr = NULL;
 
-	if (accounting_enforce != ACCOUNTING_ENFORCE_WITH_LIMITS
+	if (!(accounting_enforce & ACCOUNTING_ENFORCE_LIMITS)
 	    || !_valid_job_assoc(job_ptr))
 		return;
 
@@ -125,7 +125,7 @@ extern void acct_policy_remove_job_submit(struct job_record *job_ptr)
 	acct_association_rec_t *assoc_ptr = NULL;
 
 	if (!job_ptr->assoc_ptr || 
-	    accounting_enforce != ACCOUNTING_ENFORCE_WITH_LIMITS)
+	    !(accounting_enforce & ACCOUNTING_ENFORCE_LIMITS))
 		return;
 
 	slurm_mutex_lock(&assoc_mgr_association_lock);
@@ -150,7 +150,7 @@ extern void acct_policy_job_begin(struct job_record *job_ptr)
 {
 	acct_association_rec_t *assoc_ptr = NULL;
 
-	if (accounting_enforce != ACCOUNTING_ENFORCE_WITH_LIMITS
+	if (!(accounting_enforce & ACCOUNTING_ENFORCE_LIMITS)
 	    || !_valid_job_assoc(job_ptr))
 		return;
 
@@ -174,7 +174,7 @@ extern void acct_policy_job_fini(struct job_record *job_ptr)
 {
 	acct_association_rec_t *assoc_ptr = NULL;
 
-	if (accounting_enforce != ACCOUNTING_ENFORCE_WITH_LIMITS
+	if (!(accounting_enforce & ACCOUNTING_ENFORCE_LIMITS)
 	    || !job_ptr->assoc_ptr)
 		return;
 
@@ -231,7 +231,7 @@ extern bool acct_policy_job_runnable(struct job_record *job_ptr)
 	}
 
 	/* now see if we are enforcing limits */
-	if (accounting_enforce != ACCOUNTING_ENFORCE_WITH_LIMITS)
+	if (!(accounting_enforce & ACCOUNTING_ENFORCE_LIMITS))
 		return true;
 
 	/* clear old state reason */
@@ -301,7 +301,7 @@ extern bool acct_policy_job_runnable(struct job_record *job_ptr)
 		/* we don't need to check submit_jobs here */
 		
 		/* FIX ME: Once we start tracking time of running jobs
-		 * we will need toupdate the amount of time we have
+		 * we will need to update the amount of time we have
 		 * used and check against that here.  When we start
 		 * keeping track of time we will also need to come up
 		 * with a way to refresh the time. 
