@@ -977,6 +977,8 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 		trig_in->group_id = child;
 	} else if (child == 0) {
 		int i;
+		bool run_as_self = (uid == getuid());
+
 		for (i=0; i<128; i++)
 			close(i);
 #ifdef SETPGRP_TWO_ARGS
@@ -985,9 +987,18 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 		setpgrp();
 #endif
 		setsid();
-		setuid(uid);
-		setgid(gid);
-		initgroups(user_name, -1);
+		if ((initgroups(user_name, gid) == -1) && !run_as_self) {
+			error("trigger: initgroups: %m");
+			exit(1);
+		}
+		if ((setgid(gid) == -1) && !run_as_self){
+			error("trigger: setgid: %m");
+			exit(1);
+		}
+		if ((setuid(uid) == -1) && !run_as_self) {
+			error("trigger: setuid: %m");
+			exit(1);
+		}
 		execl(program, arg0, arg1, NULL);
 		exit(1);
 	} else
