@@ -1,6 +1,6 @@
 /*****************************************************************************\
  * src/slurmd/slurmstepd/ulimits.c - set user limits for job
- * $Id: ulimits.c 13672 2008-03-19 23:10:58Z jette $
+ * $Id: ulimits.c 15505 2008-10-27 17:39:44Z jette $
  *****************************************************************************
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -192,8 +192,8 @@ _set_limit(char **env, slurm_rlimits_info_t *rli)
 	 * Nothing to do if the rlimit won't change
 	 */
 	if (r.rlim_cur == (rlim_t) env_value) {
-		debug2( "_set_limit: %s setrlimit %s is unnecessary (same val)",
-			u_req_propagate?"user":"conf", rlimit_name );
+		debug2( "_set_limit: %s setrlimit %s no change in value: %u",
+			u_req_propagate?"user":"conf", rlimit_name, r.rlim_cur);
 		return SLURM_SUCCESS;
 	}
 
