@@ -103,10 +103,7 @@ static int _set_wckey_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if(!end && !strncasecmp(argv[i], "where",
-					MAX(command_len, 5))) {
-			continue;
-		} else if(!end && !strncasecmp(argv[i], "all_clusters", 
+		if(!end && !strncasecmp(argv[i], "all_clusters", 
 					       MAX(command_len, 1))) {
 			local_cluster_flag = 1;
 		} else if(!end && !strncasecmp(argv[i], "withdeleted",
@@ -115,8 +112,6 @@ static int _set_wckey_cond(int *start, int argc, char *argv[],
 			set = 1;
 		} else if(!end
 			  || !strncasecmp (argv[i], "WCKeys",
-					   MAX(command_len, 3))
-			  || !strncasecmp (argv[i], "Names",
 					   MAX(command_len, 3))) {
 			if(!wckey_cond->name_list)
 				wckey_cond->name_list =
@@ -212,13 +207,7 @@ static int _set_assoc_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if (!strncasecmp (argv[i], "Set", MAX(command_len, 3))) {
-			i--;
-			break;
-		} else if(!end && !strncasecmp(argv[i], "where", 
-					       MAX(command_len, 5))) {
-			continue;
-		} else if(!end && !strncasecmp(argv[i], "all_clusters", 
+		if(!end && !strncasecmp(argv[i], "all_clusters", 
 					       MAX(command_len, 1))) {
 			local_cluster_flag = 1;
 		} else if (!end && !strncasecmp (argv[i], "Tree",
@@ -319,19 +308,11 @@ static int _set_cluster_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if (!strncasecmp (argv[i], "Set", MAX(command_len, 3))) {
-			i--;
-			break;
-		} else if(!end && !strncasecmp(argv[i], "where",
-					       MAX(command_len, 5))) {
-			continue;
-		} else if(!end && !strncasecmp(argv[i], "all_clusters",
+		if(!end && !strncasecmp(argv[i], "all_clusters",
 					       MAX(command_len, 1))) {
 			local_cluster_flag = 1;
 		} else if(!end
 			  || !strncasecmp (argv[i], "Clusters",
-					   MAX(command_len, 1))
-			  || !strncasecmp (argv[i], "Names", 
 					   MAX(command_len, 1))) {
 			slurm_addto_char_list(cluster_cond->cluster_list,
 					      argv[i]+end);
