@@ -73,7 +73,9 @@ enum {
 	SORTID_RAMDISKIMAGE,
 	SORTID_STATE,
 	SORTID_UPDATED, 
+#ifdef HAVE_BGL
 	SORTID_USE,
+#endif
 	SORTID_USER,
 	SORTID_CNT
 };
@@ -195,6 +197,7 @@ static int _in_slurm_partition(int *part_inx, int *bp_inx)
 
 static char* _convert_conn_type(enum connection_type conn_type)
 {
+#ifdef HAVE_BG
 	switch (conn_type) {
 	case (SELECT_MESH):
 		return "MESH";
@@ -204,7 +207,24 @@ static char* _convert_conn_type(enum connection_type conn_type)
 		return "SMALL";
 	case (SELECT_NAV):
 		return "NAV";
+#ifndef HAVE_BGL
+	case SELECT_HTC_S:
+		return "HTC_S";
+		break;
+	case SELECT_HTC_D:
+		return "HTC_D";
+		break;
+	case SELECT_HTC_V:
+		return "HTC_V";
+		break;
+	case SELECT_HTC_L:
+		return "HTC_L";
+		break;
+#endif
+	default:
+		return "?";
 	}
+#endif
 	return "?";
 }
 
