@@ -1,7 +1,8 @@
 /****************************************************************************\
  *  grid.c - put display grid info here
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2004-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>, et. al.
  *  LLNL-CODE-402394.
@@ -48,6 +49,16 @@ int sview_colors_cnt = 20;
 
 GStaticMutex blinking_mutex = G_STATIC_MUTEX_INIT;
 
+#ifdef HAVE_3D
+static int _coord(char coord)
+{
+	if ((coord >= '0') && (coord <= '9'))
+		return (coord - '0');
+	if ((coord >= 'A') && (coord <= 'Z'))
+		return (coord - 'A');
+	return -1;
+}
+#endif
 
 static void _open_node(GtkWidget *widget, GdkEventButton *event, 
 		       grid_button_t *grid_button)
@@ -540,7 +551,7 @@ extern void add_extra_cr_buttons(List *button_list, node_info_t *node_ptr)
 extern void put_buttons_in_table(GtkTable *table, List button_list)
 {
 	int table_x=0, table_y=0;
-#ifndef HAVE_BG
+#ifndef HAVE_3D
 	int coord_x=0, coord_y=0;
 #endif
 	grid_button_t *grid_button = NULL;
@@ -549,7 +560,7 @@ extern void put_buttons_in_table(GtkTable *table, List button_list)
 	
 	list_sort(button_list, (ListCmpF) _sort_button_inx);
 	
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	node_count = DIM_SIZE[X];
 	table_x = DIM_SIZE[X] + DIM_SIZE[Z];
 	table_y = (DIM_SIZE[Z] * DIM_SIZE[Y]) + DIM_SIZE[Y];
@@ -568,7 +579,7 @@ extern void put_buttons_in_table(GtkTable *table, List button_list)
 	gtk_table_resize(table, table_y, table_x);
 	itr = list_iterator_create(button_list);
 	while((grid_button = list_next(itr))) {
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 		grid_button->table = table;
 		gtk_table_attach(table, grid_button->button,
 				 grid_button->table_x, 
@@ -654,22 +665,15 @@ extern int get_system_stats(GtkTable *table)
 extern int setup_grid_table(GtkTable *table, List button_list, List node_list)
 {
 	int error_code = SLURM_SUCCESS;
-	int x=0, table_x=0, table_y=0;
-	int coord_x=0, coord_y=0, i=0;
+	int table_x=0, table_y=0;
+	int coord_x=0, coord_y=0, inx=0;
 	grid_button_t *grid_button = NULL;
 	int node_count = 0;
 	ListIterator itr = NULL;
 	sview_node_info_t *sview_node_info_ptr = NULL;
-#ifdef HAVE_BG
-	int y=0, z=0, x_offset=0, y_offset=0;
-#endif
-
-	if(!node_list) {
-		g_print("setup_grid_table: no node_list given\n");
-		return SLURM_ERROR;
-	}
-	
-#ifdef HAVE_BG
+#ifdef HAVE_3D
+	int default_y_offset= (DIM_SIZE[Z] * DIM_SIZE[Y]) 
+		+ (DIM_SIZE[Y] - DIM_SIZE[Z]);
 	node_count = DIM_SIZE[X];
 	table_x = DIM_SIZE[X] + DIM_SIZE[Z];
 	table_y = (DIM_SIZE[Z] * DIM_SIZE[Y]) + DIM_SIZE[Y];
@@ -686,81 +690,49 @@ extern int setup_grid_table(GtkTable *table, List button_list, List node_list)
 	table_y++;
 #endif
 
+	if(!node_list) {
+		g_print("setup_grid_table: no node_list given\n");
+		return SLURM_ERROR;
+	}
+	
 	gtk_table_resize(table, table_y, table_x);
 	itr = list_iterator_create(node_list);
-#ifdef HAVE_BG
-	/* ok this is going to look much different than smap since we
-	 * get the nodes from the controller going up from the Z dim
-	 * instead of laying these out in a nice X fashion
-	 */
-	for (x=0; x<DIM_SIZE[X]; x++) {
-		y_offset = (DIM_SIZE[Z] * DIM_SIZE[Y]);
-		for (y=0; y<DIM_SIZE[Y]; y++) {
-			coord_y = y_offset - y;
-			x_offset = DIM_SIZE[Z] - 1;
-			for (z=0; z<DIM_SIZE[Z]; z++){
-				coord_x = x + x_offset;
-				
-				grid_button = xmalloc(sizeof(grid_button_t));
-				grid_button->inx = i++;
-				grid_button->table = table;
-				grid_button->table_x = coord_x;
-				grid_button->table_y = coord_y;
-				grid_button->button = gtk_button_new();
-				grid_button->tip = gtk_tooltips_new();
-				if(!(sview_node_info_ptr = list_next(itr))) {
-					g_print("no node for this "
-						"inx %d!!!!\n",
-						grid_button->inx);
-					goto end_it;
-				}
-			
-				grid_button->node_name = xstrdup(
-					sview_node_info_ptr->node_ptr->name);
-				
-				gtk_tooltips_set_tip(grid_button->tip,
-						     grid_button->button,
-						     grid_button->node_name,
-						     "click for node stats");
-				gtk_widget_set_size_request(
-					grid_button->button, 10, 10);
-				g_signal_connect(G_OBJECT(grid_button->button),
-						 "button-press-event",
-						 G_CALLBACK(_open_node),
-						 grid_button);
-				list_append(button_list, grid_button);
-								
-				gtk_table_attach(table, grid_button->button,
-						 coord_x, (coord_x+1),
-						 coord_y, (coord_y+1),
-						 GTK_SHRINK, GTK_SHRINK,
-						 1, 1);
-				
-				coord_y++;
-				x_offset--;
-			}
-			y_offset -= DIM_SIZE[Z];			
+	while((sview_node_info_ptr = list_next(itr))) {
+#ifdef HAVE_3D
+		int i = strlen(sview_node_info_ptr->node_ptr->name);
+		int x=0, y=0, z=0, y_offset=0;
+		/* On 3D system we need to translate a 3D space to
+		   a 2D space and make it appear 3D.  So we get the
+		   coords of each node in xyz format and apply an x
+		   and y offset to get a coord_x and coord_y.  This is
+		   not needed for linear systems since they can be
+		   laid out in any fashion 
+		*/
+		if (i < 4) {
+			g_error("bad node name %s\n",
+				sview_node_info_ptr->node_ptr->name);
+			goto end_it;
+		} else {
+			x = _coord(sview_node_info_ptr->node_ptr->name[i-3]);
+			y = _coord(sview_node_info_ptr->node_ptr->name[i-2]);
+			z = _coord(sview_node_info_ptr->node_ptr->name[i-1]);
 		}
-		gtk_table_set_row_spacing(table, coord_y-1, 5);
-	}
-#else
-	for (x=0; x<node_count; x++) {
+		coord_x = (x + (DIM_SIZE[Z] - 1)) - z;
+
+		y_offset = default_y_offset - (DIM_SIZE[Z] * y);
+		coord_y = (y_offset - y) + z;
+#endif
 		grid_button = xmalloc(sizeof(grid_button_t));
-		grid_button->inx = i++;
+		grid_button->inx = inx++;
 		grid_button->table = table;
 		grid_button->table_x = coord_x;
 		grid_button->table_y = coord_y;
-		
 		grid_button->button = gtk_button_new();
 		grid_button->tip = gtk_tooltips_new();
-		if(!(sview_node_info_ptr = list_next(itr))) {
-			g_print("no node for this "
-				"inx %d!!!!\n",
-				grid_button->inx);
-			goto end_it;
-		}
-		grid_button->node_name = xstrdup(
-			sview_node_info_ptr->node_ptr->name);
+
+		grid_button->node_name =
+			xstrdup(sview_node_info_ptr->node_ptr->name);
+
 		gtk_tooltips_set_tip(grid_button->tip,
 				     grid_button->button,
 				     grid_button->node_name,
@@ -773,33 +745,38 @@ extern int setup_grid_table(GtkTable *table, List button_list, List node_list)
 		list_append(button_list, grid_button);
 		
 		gtk_table_attach(table, grid_button->button,
-				 coord_x, (coord_x+1), coord_y, (coord_y+1),
+				 coord_x, (coord_x+1),
+				 coord_y, (coord_y+1),
 				 GTK_SHRINK, GTK_SHRINK,
-				 1, 1);
-		
-		coord_x++;
-			
+				 1, 1);		
+#ifndef HAVE_3D
+		/* On linear systems we just up the x_coord until we
+		   hit the side of the table and then incrememnt the
+		   coord_y.  We add space inbetween each 10th row.
+		*/
+		coord_x++;			
 		if(coord_x == table_x) {
 			coord_x = 0;
 			coord_y++;
-			if(!(coord_y%10)) {
-				gtk_table_set_row_spacing(table,
-							  coord_y-1, 5);
-			}
-			
+			if(!(coord_y%10)) 
+				gtk_table_set_row_spacing(table, coord_y-1, 5);
 		}
 		
 		if(coord_y == table_y)
 			break;
 	
-		if(coord_x && !(coord_x%10)) {
-			gtk_table_set_col_spacing(table,
-						  coord_x-1, 5);
-		}
-	}
+		if(coord_x && !(coord_x%10)) 
+			gtk_table_set_col_spacing(table, coord_x-1, 5);
 #endif
-	
+	}
+
+#ifdef HAVE_3D
+	/* This is needed to get the correct width of the grid
+	   window.  If it is not given then we get a really narrow
+	   window. */
+	gtk_table_set_row_spacing(table, coord_y-1, 1);
 end_it:
+#endif
 	list_iterator_destroy(itr);
 	list_sort(button_list, (ListCmpF) _sort_button_inx);
 	
