@@ -2,7 +2,8 @@
  *  job_info.c - Functions related to job display 
  *  mode of sview.
  *****************************************************************************
- *  Copyright (C) 2004-2006 The Regents of the University of California.
+ *  Copyright (C) 2004-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
  *
@@ -137,6 +138,7 @@ enum {
 	SORTID_TIME,
 	SORTID_TIMELIMIT,
 	SORTID_TMP_DISK,
+	SORTID_WCKEY,
 	SORTID_UPDATED,
 	SORTID_USER, 
 	SORTID_CNT
@@ -182,6 +184,8 @@ static display_data_t display_data_job[] = {
 	 create_model_job, admin_edit_job},
 	{G_TYPE_STRING, SORTID_GROUP, "Group", FALSE, EDIT_NONE, refresh_job,
 	 create_model_job, admin_edit_job},
+	{G_TYPE_STRING, SORTID_WCKEY, "WCKey", FALSE, EDIT_TEXTBOX, refresh_job,
+	 create_model_job, admin_edit_job},
 	{G_TYPE_STRING, SORTID_NAME, "Name", TRUE, EDIT_TEXTBOX, refresh_job,
 	 create_model_job, admin_edit_job},
 	{G_TYPE_STRING, SORTID_STATE, "State", TRUE, EDIT_NONE, refresh_job,
@@ -559,6 +563,9 @@ static const char *_set_job_msg(job_desc_msg_t *job_msg, const char *new_text,
 	char* geometry_tmp = xstrdup(new_text);
 	char* original_ptr = geometry_tmp;
 #endif
+	/* need to clear errno here (just in case) */
+	errno = 0;
+
 	if(!job_msg)
 		return NULL;
 	
@@ -656,6 +663,10 @@ static const char *_set_job_msg(job_desc_msg_t *job_msg, const char *new_text,
 		job_msg->name = xstrdup(new_text);
 		type = "name";
 		break;
+	case SORTID_WCKEY:		
+		xstrfmtcat(job_msg->name, "\"%s", new_text);
+		type = "wckey";
+		break;
 	case SORTID_SHARED:
 		if (!strcasecmp(new_text, "yes")) {
 			job_msg->shared = 1;
@@ -832,7 +843,7 @@ static const char *_set_job_msg(job_desc_msg_t *job_msg, const char *new_text,
 		break;
 #endif
 	case SORTID_START_TIME:
-		job_msg->begin_time = parse_time((char *)new_text);
+		job_msg->begin_time = parse_time((char *)new_text, 0);
 		type = "start time";
 		break;
 	default:
@@ -1028,7 +1039,7 @@ static void _layout_job_record(GtkTreeView *treeview,
 			       sview_job_info_t *sview_job_info_ptr, 
 			       int update)
 {
-	char *nodes = NULL;
+	char *nodes = NULL, *uname = NULL, *jname = NULL, *wckey = NULL;
 	char tmp_char[50];
 	time_t now_time = time(NULL);
 	job_info_t *job_ptr = sview_job_info_ptr->job_ptr;
@@ -1039,10 +1050,27 @@ static void _layout_job_record(GtkTreeView *treeview,
 	GtkTreeStore *treestore = 
 		GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
 
+	if (job_ptr->name && job_ptr->name[0]) {
+		char *temp = NULL;
+		/* first set the jname to the job_ptr->name */
+		jname = xstrdup(job_ptr->name);
+		/* then grep for " since that is the delimiter for
+		   the wckey */
+		if((temp = strchr(jname, '\"'))) {
+			/* if we have a wckey set the " to NULL to
+			 * end the jname */
+			temp[0] = '\0';
+			/* increment and copy the remainder */
+			temp++;
+			wckey = xstrdup(temp);
+		}
+	}
+	
+
 	if(!treestore)
 		return;
 	if(!job_ptr->nodes || !strcasecmp(job_ptr->nodes,"waiting...")) {
-		sprintf(tmp_char,"0:00:00");
+		sprintf(tmp_char,"00:00:00");
 		nodes = "waiting...";
 	} else {
 		if (job_ptr->job_state == JOB_SUSPENDED)
@@ -1128,10 +1156,11 @@ static void _layout_job_record(GtkTreeView *treeview,
 					   sizeof(tmp_char), 
 					   SELECT_PRINT_BG_ID));
 #endif
+	uname = uid_to_string((uid_t)job_ptr->user_id);
 	add_display_treestore_line(update, treestore, &iter, 
 				   find_col_name(display_data_job,
-						 SORTID_USER), 
-				   uid_to_string((uid_t)job_ptr->user_id));
+						 SORTID_USER), uname);
+	xfree(uname);
 	group_info = getgrgid((gid_t) job_ptr->group_id );
 	if ( group_info && group_info->gr_name[ 0 ] ) {
 		snprintf(tmp_char, sizeof(tmp_char), "%s",
@@ -1148,7 +1177,12 @@ static void _layout_job_record(GtkTreeView *treeview,
 	add_display_treestore_line(update, treestore, &iter, 
 				   find_col_name(display_data_job,
 						 SORTID_NAME), 
-				   job_ptr->name);
+				   jname);
+	
+	add_display_treestore_line(update, treestore, &iter, 
+				   find_col_name(display_data_job,
+						 SORTID_WCKEY), 
+				   wckey);
 	
 	sprintf(tmp_char, "%u", job_ptr->priority);
 	add_display_treestore_line(update, treestore, &iter, 
@@ -1413,13 +1447,15 @@ static void _layout_job_record(GtkTreeView *treeview,
 				   find_col_name(display_data_job,
 						 SORTID_COMMENT),
 				   job_ptr->comment);
+	xfree(jname);
+	xfree(wckey);
 }
 
 static void _update_job_record(sview_job_info_t *sview_job_info_ptr, 
 			       GtkTreeStore *treestore,
 			       GtkTreeIter *iter)
 {
-	char *nodes = NULL;
+	char *nodes = NULL, *uname = NULL, *jname = NULL, *wckey = NULL;
 	char tmp_char[50];
 	time_t now_time = time(NULL);
 	GtkTreeIter step_iter;
@@ -1427,10 +1463,26 @@ static void _update_job_record(sview_job_info_t *sview_job_info_ptr,
 	job_info_t *job_ptr = sview_job_info_ptr->job_ptr;
 	struct group *group_info = NULL;
 	uint16_t term_sig = 0;
+     
+	if (job_ptr->name && job_ptr->name[0]) {
+		char *temp = NULL;
+		/* first set the jname to the job_ptr->name */
+		jname = xstrdup(job_ptr->name);
+		/* then grep for " since that is the delimiter for
+		   the wckey */
+		if((temp = strchr(jname, '\"'))) {
+			/* if we have a wckey set the " to NULL to
+			 * end the jname */
+			temp[0] = '\0';
+			/* increment and copy the remainder */
+			temp++;
+			wckey = xstrdup(temp);
+		}
+	}
 	
 	gtk_tree_store_set(treestore, iter, SORTID_UPDATED, 1, -1);
 	if(!job_ptr->nodes || !strcasecmp(job_ptr->nodes,"waiting...")) {
-		sprintf(tmp_char,"0:00:00");
+		sprintf(tmp_char,"00:00:00");
 		nodes = "waiting...";
 	} else {
 		if (job_ptr->job_state == JOB_SUSPENDED)
@@ -1558,9 +1610,10 @@ static void _update_job_record(sview_job_info_t *sview_job_info_ptr,
 				   SELECT_PRINT_RAMDISK_IMAGE), -1);
 	
 #endif
+	uname = uid_to_string((uid_t)job_ptr->user_id);
 	gtk_tree_store_set(treestore, iter, 
-			   SORTID_USER, 
-			   uid_to_string((uid_t)job_ptr->user_id), -1);
+			   SORTID_USER, uname, -1);
+	xfree(uname);
 	group_info = getgrgid((gid_t) job_ptr->group_id );
 	if ( group_info && group_info->gr_name[ 0 ] ) {
 		snprintf(tmp_char, sizeof(tmp_char), "%s",
@@ -1573,7 +1626,8 @@ static void _update_job_record(sview_job_info_t *sview_job_info_ptr,
 			   SORTID_GROUP, 
 			   tmp_char, -1);
 		
-	gtk_tree_store_set(treestore, iter, SORTID_NAME, job_ptr->name, -1);
+	gtk_tree_store_set(treestore, iter, SORTID_NAME, jname, -1);
+	gtk_tree_store_set(treestore, iter, SORTID_WCKEY, wckey, -1);
 	gtk_tree_store_set(treestore, iter, 
 			   SORTID_STATE, 
 			   job_state_string(job_ptr->job_state), -1);
@@ -1711,6 +1765,8 @@ static void _update_job_record(sview_job_info_t *sview_job_info_ptr,
 		_update_info_step(sview_job_info_ptr, 
 				  GTK_TREE_MODEL(treestore), NULL, iter);
 		
+	xfree(jname);
+	xfree(wckey);
 	return;
 }
 
@@ -1718,7 +1774,7 @@ static void _layout_step_record(GtkTreeView *treeview,
 				job_step_info_t *step_ptr,
 				int update)
 {
-	char *nodes = NULL;
+	char *nodes = NULL, *uname;
 	char tmp_char[50];
 	char tmp_time[50];
 	time_t now_time = time(NULL);
@@ -1731,7 +1787,7 @@ static void _layout_step_record(GtkTreeView *treeview,
 		return;
 	if(!step_ptr->nodes 
 	   || !strcasecmp(step_ptr->nodes,"waiting...")) {
-		sprintf(tmp_time,"0:00:00");
+		sprintf(tmp_time,"00:00:00");
 		nodes = "waiting...";
 		state = JOB_PENDING;
 	} else {
@@ -1782,10 +1838,11 @@ static void _layout_step_record(GtkTreeView *treeview,
 /* 				   sizeof(tmp_char),  */
 /* 				   SELECT_PRINT_BG_ID)); */
 /* #endif */
+	uname = uid_to_string((uid_t)step_ptr->user_id);
 	add_display_treestore_line(update, treestore, &iter, 
 				   find_col_name(display_data_job,
-						 SORTID_USER), 
-				   uid_to_string((uid_t)step_ptr->user_id));
+						 SORTID_USER), uname);
+	xfree(uname);
 	add_display_treestore_line(update, treestore, &iter, 
 				   find_col_name(display_data_job,
 						 SORTID_NAME),
@@ -1813,7 +1870,7 @@ static void _update_step_record(job_step_info_t *step_ptr,
 				GtkTreeStore *treestore,
 				GtkTreeIter *iter)
 {
-	char *nodes = NULL;
+	char *nodes = NULL, *uname = NULL;
 	char tmp_char[50];
 	char tmp_time[50];
 	time_t now_time = time(NULL);
@@ -1822,7 +1879,7 @@ static void _update_step_record(job_step_info_t *step_ptr,
 	gtk_tree_store_set(treestore, iter, SORTID_UPDATED, 1, -1);
 	if(!step_ptr->nodes 
 	   || !strcasecmp(step_ptr->nodes,"waiting...")) {
-		sprintf(tmp_char,"0:00:00");
+		sprintf(tmp_char,"00:00:00");
 		nodes = "waiting...";
 		state = JOB_PENDING;
 	} else {
@@ -1862,9 +1919,10 @@ static void _update_step_record(job_step_info_t *step_ptr,
 /* 				   sizeof(tmp_char),  */
 /* 				   SELECT_PRINT_BG_ID), -1); */
 /* #endif */
+	uname = uid_to_string((uid_t)step_ptr->user_id);
 	gtk_tree_store_set(treestore, iter, 
-			   SORTID_USER, 
-			   uid_to_string((uid_t)step_ptr->user_id), -1);
+			   SORTID_USER, uname, -1);
+	xfree(uname);
 	gtk_tree_store_set(treestore, iter, 
 			   SORTID_NAME, step_ptr->name, -1);
 		
@@ -2540,7 +2598,7 @@ extern void admin_edit_job(GtkCellRendererText *cell,
 	}
 	
 	type = _set_job_msg(job_msg, new_text, column);
-	if(errno)
+	if(errno) 
 		goto print_error;
 	
 	if(got_edit_signal) {
@@ -2554,16 +2612,15 @@ extern void admin_edit_job(GtkCellRendererText *cell,
 
 	if(old_text && !strcmp(old_text, new_text)) {
 		temp = g_strdup_printf("No change in value.");
-		display_edit_note(temp);
-		g_free(temp);	
 	} else if(slurm_update_job(job_msg) == SLURM_SUCCESS) {
 		gtk_tree_store_set(treestore, &iter, column, new_text, -1);
 		temp = g_strdup_printf("Job %d %s changed to %s",
 				       job_msg->job_id,
 				       type,
 				       new_text);
-		display_edit_note(temp);
-		g_free(temp);
+	} else if(errno == ESLURM_DISABLED) {
+		temp = g_strdup_printf(
+			"Can only edit %s on pending jobs.", type);
 	} else {
 	print_error:
 		temp = g_strdup_printf("Job %d %s can't be "
@@ -2571,9 +2628,10 @@ extern void admin_edit_job(GtkCellRendererText *cell,
 				       job_msg->job_id,
 				       type,
 				       new_text);
-		display_edit_note(temp);
-		g_free(temp);
 	}
+	
+	display_edit_note(temp);
+	g_free(temp);
 
 no_input:
 	slurm_free_job_desc_msg(job_msg);
@@ -2720,10 +2778,10 @@ extern void specific_info_job(popup_info_t *popup_win)
 	sview_job_info_t *sview_job_info_ptr = NULL;
 	job_info_t *job_ptr = NULL;	
 	ListIterator itr = NULL;
-	char name[30];
+	char name[30], *uname = NULL;
 	char *host = NULL, *host2 = NULL;
 	hostlist_t hostlist = NULL;
-	int found = 0;
+	int found = 0, name_diff;
 	
 	if(!spec_info->display_widget)
 		setup_popup_info(popup_win, display_data_job, SORTID_CNT);
@@ -2809,7 +2867,7 @@ display_it:
 		list_destroy(popup_win->grid_button_list);
 	}	       
 	
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	popup_win->grid_button_list = copy_main_button_list();
 #else
 	popup_win->grid_button_list = list_create(destroy_grid_button);
@@ -2863,8 +2921,11 @@ display_it:
 			case SEARCH_JOB_USER:
 				if(!search_info->gchar_data)
 					continue;
-				if(strcmp(uid_to_string(job_ptr->user_id),
-					  search_info->gchar_data))
+				uname = uid_to_string(job_ptr->user_id);
+				name_diff = strcmp(uname, 
+						   search_info->gchar_data);
+				xfree(uname);
+				if(name_diff)
 					continue;
 				break;
 			case SEARCH_JOB_STATE:
@@ -2923,7 +2984,7 @@ display_it:
 		list_push(send_info_list, sview_job_info_ptr);
 		j=0;
 		while(job_ptr->node_inx[j] >= 0) {
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 			change_grid_color(
 				popup_win->grid_button_list,
 				job_ptr->node_inx[j],
@@ -3258,6 +3319,11 @@ extern void admin_job(GtkTreeModel *model, GtkTreeIter *iter, char *type)
 				tmp_char_ptr = g_strdup_printf(
 					"Job %u updated successfully",
 					jobid);
+			} else if(errno == ESLURM_DISABLED) {
+				tmp_char_ptr = g_strdup_printf(
+					"Can't edit that part of non-pending "
+					"job %u.",
+					jobid);
 			} else {
 				tmp_char_ptr = g_strdup_printf(
 					"Problem updating job %u.",
