@@ -197,10 +197,13 @@ static display_data_t options_data_part[] = {
 static void _update_nodes_for_bg(int node_scaling,
 				 node_info_msg_t *node_msg,
 				 bg_info_record_t *bg_info_record);
+/* ERROR_STATE must be last since that will affect the state of the rest of the
+   midplane.
+*/
 enum {
 	SVIEW_BG_IDLE_STATE,
-	SVIEW_BG_ERROR_STATE,
-	SVIEW_BG_ALLOC_STATE
+	SVIEW_BG_ALLOC_STATE,
+	SVIEW_BG_ERROR_STATE
 };
 #endif
 
@@ -384,7 +387,9 @@ static const char *_set_part_msg(update_part_msg_t *part_msg,
 {
 	char *type = NULL;
 	int temp_int = 0;
-	
+
+	errno = 0;
+
 	if(!part_msg)
 		return NULL;
 	
@@ -1492,6 +1497,18 @@ static List _create_part_info_list(partition_info_msg_t *part_info_ptr,
 				int norm = 0;
 				switch(j) {
 				case SVIEW_BG_IDLE_STATE:
+					/* check to see if the node is
+					 * down if so just report the
+					 * whole thing is down and break
+					 * out.
+					 */
+					if((node_ptr->node_state 
+					    & NODE_STATE_BASE)
+					   == NODE_STATE_DOWN) {
+						norm = 1;
+						break;
+					}
+
 					/* get the idle node count if
 					 * we don't have any error or
 					 * allocated nodes then we set
@@ -1503,29 +1520,35 @@ static List _create_part_info_list(partition_info_msg_t *part_info_ptr,
 						 + node_ptr->used_cpus);
 					if(node_ptr->threads == node_scaling)
 						norm = 1;
-					else
-						node_ptr->node_state =
+					else {
+						node_ptr->node_state &=
+							NODE_STATE_FLAGS;
+						node_ptr->node_state |=
 							NODE_STATE_IDLE;
-					
-					break;
-				case SVIEW_BG_ERROR_STATE:
-					/* get the error node count */
-					if(!node_ptr->cores) 
-						continue;
-					node_ptr->node_state |= 
-						NODE_STATE_DRAIN;
-					node_ptr->threads = node_ptr->cores;
+					}
 					break;
 				case SVIEW_BG_ALLOC_STATE:
 					/* get the allocated node count */
 					if(!node_ptr->used_cpus) 
 						continue;
-					node_ptr->node_state =
+					node_ptr->node_state &=
+						NODE_STATE_FLAGS;
+					node_ptr->node_state |=
 						NODE_STATE_ALLOCATED;
 					
 					node_ptr->threads =
 						node_ptr->used_cpus;
 					break;
+				case SVIEW_BG_ERROR_STATE:
+					/* get the error node count */
+					if(!node_ptr->cores) 
+						continue;
+					node_ptr->node_state &=
+						NODE_STATE_FLAGS;
+					node_ptr->node_state |= 
+						NODE_STATE_DRAIN;
+					node_ptr->threads = node_ptr->cores;
+					break;
 				default:
 					error("unknown state");
 					break;
@@ -1885,8 +1908,6 @@ extern void admin_edit_part(GtkCellRendererText *cell,
 	if(column != SORTID_STATE && column != SORTID_FEATURES ) {
 		if(old_text && !strcmp(old_text, new_text)) {
 			temp = g_strdup_printf("No change in value.");
-			display_edit_note(temp);
-			g_free(temp);	
 		} else if(slurm_update_partition(part_msg) == SLURM_SUCCESS) {
 			gtk_tree_store_set(treestore, &iter, column,
 					   new_text, -1);
@@ -1894,8 +1915,6 @@ extern void admin_edit_part(GtkCellRendererText *cell,
 					       part_msg->name,
 					       type,
 					       new_text);
-			display_edit_note(temp);
-			g_free(temp);
 		} else {
 		print_error:
 			temp = g_strdup_printf("Partition %s %s can't be "
@@ -1903,9 +1922,9 @@ extern void admin_edit_part(GtkCellRendererText *cell,
 					       part_msg->name,
 					       type,
 					       new_text);
-			display_edit_note(temp);
-			g_free(temp);
 		}
+		display_edit_note(temp);
+		g_free(temp);	
 	}
 no_input:
 	slurm_free_update_part_msg(part_msg);
@@ -2188,7 +2207,7 @@ display_it:
 		list_destroy(popup_win->grid_button_list);
 	}	       
 	
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 	popup_win->grid_button_list = copy_main_button_list();
 #else
 	popup_win->grid_button_list = list_create(destroy_grid_button);
@@ -2249,7 +2268,7 @@ display_it:
 		list_push(send_info_list, sview_part_info_ptr);
 		j=0;
 		while(part_ptr->node_inx[j] >= 0) {
-#ifdef HAVE_BG
+#ifdef HAVE_3D
 			change_grid_color(
 				popup_win->grid_button_list,
 				part_ptr->node_inx[j],
