@@ -1,9 +1,9 @@
 /*****************************************************************************\
- *  test1.34.prog.c - Sleep for a second and terminate with SIGSEGV
+ *  test7.4.prog.c - Test of TotalView operation with SLURM.
  *****************************************************************************
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2004 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  Written by Morris Jette <jette1@llnl.gov>
+ *  Written by Dong Ang <dahn@llnl.gov>
  *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
@@ -23,11 +23,40 @@
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
-int main (int argc, char *argv[])
-{
-	char *buffer;
 
-	buffer = (char *) 0;
-	buffer[1] = 'a';
-	return;
+#include <stdio.h>
+#include <mpi.h>
+
+#define COMM_TAG 1000
+
+static void pass_its_neighbor(const int rank, const int size, const int* buf)
+{
+	MPI_Request request[2];
+	MPI_Status status[2];
+
+	MPI_Irecv((void *)buf, 1, MPI_INT, ((rank+size-1)%size), COMM_TAG,
+		MPI_COMM_WORLD, &request[0]);
+	MPI_Isend((void *)&rank, 1, MPI_INT, ((rank+1)%size), COMM_TAG,
+		MPI_COMM_WORLD, &request[1]);
+	MPI_Waitall(2, request, status);
+
+	fprintf(stdout, "Rank[%d] I just received msg from Rank %d\n", 
+		rank, *buf);
 }
+
+int main(int argc, char * argv[])
+{
+	int size, rank,buf;
+
+	MPI_Init(&argc, &argv);
+	MPI_Comm_size(MPI_COMM_WORLD, &size);
+	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+
+	buf = rank;	/* we only pass rank */
+
+	pass_its_neighbor(rank, size, &buf);
+
+	MPI_Finalize();
+	return 0;
+}
+
